# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from openstack.accelerator.v2._proxy import Proxy


class AcceleratorCloudMixin:
    accelerator: Proxy

    def list_deployables(self, filters=None):
        """List all available deployables.

        :param filters: (optional) dict of filter conditions to push down
        :returns: A list of accelerator ``Deployable`` objects.
        """
        # Translate None from search interface to empty {} for kwargs below
        if not filters:
            filters = {}
        return list(self.accelerator.deployables(**filters))

    def list_devices(self, filters=None):
        """List all devices.

        :param filters: (optional) dict of filter conditions to push down
        :returns: A list of accelerator ``Device`` objects.
        """
        # Translate None from search interface to empty {} for kwargs below
        if not filters:
            filters = {}
        return list(self.accelerator.devices(**filters))

    def list_device_profiles(self, filters=None):
        """List all device_profiles.

        :param filters: (optional) dict of filter conditions to push down
        :returns: A list of accelerator ``DeviceProfile`` objects.
        """
        # Translate None from search interface to empty {} for kwargs below
        if not filters:
            filters = {}
        return list(self.accelerator.device_profiles(**filters))

    def create_device_profile(self, attrs):
        """Create a device_profile.

        :param attrs: The info of device_profile to be created.
        :returns: An accelerator ``DeviceProfile`` objects.
        """
        return self.accelerator.create_device_profile(**attrs)

    def delete_device_profile(self, name_or_id, filters):
        """Delete a device_profile.

        :param name_or_id: The name or uuid of the device profile to be
            deleted.
        :param filters: dict of filter conditions to push down
        :returns: True if delete succeeded, False otherwise.
        """
        device_profile = self.accelerator.get_device_profile(
            name_or_id,
            filters,
        )
        if device_profile is None:
            self.log.debug(
                "device_profile %s not found for deleting",
                name_or_id,
            )
            return False

        self.accelerator.delete_device_profile(device_profile=device_profile)

        return True

    def list_accelerator_requests(self, filters=None):
        """List all accelerator_requests.

        :param filters: (optional) dict of filter conditions to push down
        :returns: A list of accelerator ``AcceleratorRequest`` objects.
        """
        # Translate None from search interface to empty {} for kwargs below
        if not filters:
            filters = {}
        return list(self.accelerator.accelerator_requests(**filters))

    def delete_accelerator_request(self, name_or_id, filters):
        """Delete a accelerator_request.

        :param name_or_id: The name or UUID of the accelerator request to
            be deleted.
        :param filters: dict of filter conditions to push down
        :returns: True if delete succeeded, False otherwise.
        """
        accelerator_request = self.accelerator.get_accelerator_request(
            name_or_id,
            filters,
        )
        if accelerator_request is None:
            self.log.debug(
                "accelerator_request %s not found for deleting",
                name_or_id,
            )
            return False

        self.accelerator.delete_accelerator_request(
            accelerator_request=accelerator_request,
        )

        return True

    def create_accelerator_request(self, attrs):
        """Create an accelerator_request.

        :param attrs: The info of accelerator_request to be created.
        :returns: An accelerator ``AcceleratorRequest`` object.
        """
        return self.accelerator.create_accelerator_request(**attrs)

    def bind_accelerator_request(self, uuid, properties):
        """Bind an accelerator to VM.

        :param uuid: The uuid of the accelerator_request to be binded.
        :param properties: The info of VM that will bind the accelerator.
        :returns: True if bind succeeded, False otherwise.
        """
        accelerator_request = self.accelerator.get_accelerator_request(uuid)
        if accelerator_request is None:
            self.log.debug(
                "accelerator_request %s not found for unbinding", uuid
            )
            return False

        return self.accelerator.update_accelerator_request(uuid, properties)

    def unbind_accelerator_request(self, uuid, properties):
        """Unbind an accelerator from VM.

        :param uuid: The uuid of the accelerator_request to be unbinded.
        :param properties: The info of VM that will unbind the accelerator.
        :returns: True if unbind succeeded, False otherwise.
        """
        accelerator_request = self.accelerator.get_accelerator_request(uuid)
        if accelerator_request is None:
            self.log.debug(
                "accelerator_request %s not found for unbinding", uuid
            )
            return False

        return self.accelerator.update_accelerator_request(uuid, properties)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from openstack.tests.unit import base


class TestGroups(base.TestCase):
    def setUp(self, cloud_config_fixture='clouds.yaml'):
        super(TestGroups, self).setUp(
            cloud_config_fixture=cloud_config_fixture
        )
        self.addCleanup(self.assert_calls)

    def get_mock_url(
        self,
        service_type='identity',
        interface='public',
        resource='groups',
        append=None,
        base_url_append='v3',
    ):
        return super(TestGroups, self).get_mock_url(
            service_type='identity',
            interface=interface,
            resource=resource,
            append=append,
            base_url_append=base_url_append,
        )

    def test_list_groups(self):
        group_data = self._get_group_data()
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(),
                    status_code=200,
                    json={'groups': [group_data.json_response['group']]},
                )
            ]
        )
        self.cloud.list_groups()

    def test_get_group(self):
        group_data = self._get_group_data()
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(),
                    status_code=200,
                    json={'groups': [group_data.json_response['group']]},
                ),
            ]
        )
        self.cloud.get_group(group_data.group_id)

    def test_delete_group(self):
        group_data = self._get_group_data()
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(append=[group_data.group_id]),
                    status_code=200,
                    json={'group': group_data.json_response['group']},
                ),
                dict(
                    method='DELETE',
                    uri=self.get_mock_url(append=[group_data.group_id]),
                    status_code=204,
                ),
            ]
        )
        self.assertTrue(self.cloud.delete_group(group_data.group_id))

    def test_create_group(self):
        domain_data = self._get_domain_data()
        group_data = self._get_group_data(domain_id=domain_data.domain_id)
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        resource='domains', append=[domain_data.domain_id]
                    ),
                    status_code=200,
                    json=domain_data.json_response,
                ),
                dict(
                    method='POST',
                    uri=self.get_mock_url(),
                    status_code=200,
                    json=group_data.json_response,
                    validate=dict(json=group_data.json_request),
                ),
            ]
        )
        self.cloud.create_group(
            name=group_data.group_name,
            description=group_data.description,
            domain=group_data.domain_id,
        )

    def test_update_group(self):
        group_data = self._get_group_data()
        # Domain ID is not sent
        group_data.json_request['group'].pop('domain_id')
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(append=[group_data.group_id]),
                    status_code=200,
                    json={'group': group_data.json_response['group']},
                ),
                dict(
                    method='PATCH',
                    uri=self.get_mock_url(append=[group_data.group_id]),
                    status_code=200,
                    json=group_data.json_response,
                    validate=dict(
                        json={
                            'group': {
                                'name': 'new_name',
                                'description': 'new_description',
                            }
                        }
                    ),
                ),
            ]
        )
        self.cloud.update_group(
            group_data.group_id, 'new_name', 'new_description'
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.block_storage.v3 import group_snapshot
from openstack.tests.unit import base

GROUP_SNAPSHOT = {
    "id": "6f519a48-3183-46cf-a32f-41815f813986",
    "group_id": "6f519a48-3183-46cf-a32f-41815f814444",
    "status": "available",
    "created_at": "2015-09-16T09:28:52.000000",
    "name": "my_group_snapshot1",
    "description": "my first group snapshot",
    "group_type_id": "7270c56e-6354-4528-8e8b-f54dee2232c8",
    "project_id": "7ccf4863071f44aeb8f141f65780c51b",
}


class TestGroupSnapshot(base.TestCase):
    def test_basic(self):
        resource = group_snapshot.GroupSnapshot()
        self.assertEqual("group_snapshot", resource.resource_key)
        self.assertEqual("group_snapshots", resource.resources_key)
        self.assertEqual("/group_snapshots", resource.base_path)
        self.assertTrue(resource.allow_create)
        self.assertTrue(resource.allow_fetch)
        self.assertTrue(resource.allow_delete)
        self.assertTrue(resource.allow_list)
        self.assertFalse(resource.allow_commit)

    def test_make_resource(self):
        resource = group_snapshot.GroupSnapshot(**GROUP_SNAPSHOT)
        self.assertEqual(GROUP_SNAPSHOT["created_at"], resource.created_at)
        self.assertEqual(GROUP_SNAPSHOT["description"], resource.description)
        self.assertEqual(GROUP_SNAPSHOT["group_id"], resource.group_id)
        self.assertEqual(
            GROUP_SNAPSHOT["group_type_id"], resource.group_type_id
        )
        self.assertEqual(GROUP_SNAPSHOT["id"], resource.id)
        self.assertEqual(GROUP_SNAPSHOT["name"], resource.name)
        self.assertEqual(GROUP_SNAPSHOT["project_id"], resource.project_id)
        self.assertEqual(GROUP_SNAPSHOT["status"], resource.status)



# Copyright (c) 2018 Red Hat, Inc.

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import ddt
from keystoneauth1 import exceptions

from openstack.tests.unit import base


@ddt.ddt
class TestPlacementRest(base.TestCase):
    def setUp(self):
        super(TestPlacementRest, self).setUp()
        self.use_placement()

    def _register_uris(self, status_code=None):
        uri = dict(
            method='GET',
            uri=self.get_mock_url(
                'placement', 'public', append=['allocation_candidates']
            ),
            json={},
        )
        if status_code is not None:
            uri['status_code'] = status_code
        self.register_uris([uri])

    def _validate_resp(self, resp, status_code):
        self.assertEqual(status_code, resp.status_code)
        self.assertEqual(
            'https://placement.example.com/allocation_candidates', resp.url
        )
        self.assert_calls()

    @ddt.data({}, {'raise_exc': False}, {'raise_exc': True})
    def test_discovery(self, get_kwargs):
        self._register_uris()
        # Regardless of raise_exc, a <400 response doesn't raise
        rs = self.cloud.placement.get('/allocation_candidates', **get_kwargs)
        self._validate_resp(rs, 200)

    @ddt.data({}, {'raise_exc': False})
    def test_discovery_err(self, get_kwargs):
        self._register_uris(status_code=500)
        # >=400 doesn't raise by default or with explicit raise_exc=False
        rs = self.cloud.placement.get('/allocation_candidates', **get_kwargs)
        self._validate_resp(rs, 500)

    def test_discovery_exc(self):
        self._register_uris(status_code=500)
        # raise_exc=True raises a ksa exception appropriate to the status code
        ex = self.assertRaises(
            exceptions.InternalServerError,
            self.cloud.placement.get,
            '/allocation_candidates',
            raise_exc=True,
        )
        self._validate_resp(ex.response, 500)

    def test_microversion_discovery(self):
        self.assertEqual(
            (1, 17), self.cloud.placement.get_endpoint_data().max_microversion
        )
        self.assert_calls()


class TestBadPlacementRest(base.TestCase):
    def setUp(self):
        self.skipTest('Need to re-add support for broken placement versions')
        super(TestBadPlacementRest, self).setUp()
        # The bad-placement.json is for older placement that was
        # missing the status field from its discovery doc. This
        # lets us show that we can talk to such a placement.
        self.use_placement(discovery_fixture='bad-placement.json')

    def _register_uris(self, status_code=None):
        uri = dict(
            method='GET',
            uri=self.get_mock_url(
                'placement', 'public', append=['allocation_candidates']
            ),
            json={},
        )
        if status_code is not None:
            uri['status_code'] = status_code
        self.register_uris([uri])

    def _validate_resp(self, resp, status_code):
        self.assertEqual(status_code, resp.status_code)
        self.assertEqual(
            'https://placement.example.com/allocation_candidates', resp.url
        )
        self.assert_calls()

    def test_discovery(self):
        self._register_uris()
        rs = self.cloud.placement.get('/allocation_candidates')
        self._validate_resp(rs, 200)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from openstack.compute.v2 import service
from openstack import exceptions
from openstack.tests.unit import base

IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    'id': IDENTIFIER,
    'binary': 'nova-compute',
    'host': 'host1',
    'status': 'enabled',
    'state': 'up',
    'zone': 'nova',
}


class TestService(base.TestCase):
    def setUp(self):
        super(TestService, self).setUp()
        self.resp = mock.Mock()
        self.resp.body = {'service': {}}
        self.resp.json = mock.Mock(return_value=self.resp.body)
        self.resp.status_code = 200
        self.resp.headers = {}
        self.sess = mock.Mock()
        self.sess.put = mock.Mock(return_value=self.resp)
        self.sess.default_microversion = '2.1'

    def test_basic(self):
        sot = service.Service()
        self.assertEqual('service', sot.resource_key)
        self.assertEqual('services', sot.resources_key)
        self.assertEqual('/os-services', sot.base_path)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_list)
        self.assertFalse(sot.allow_fetch)

        self.assertDictEqual(
            {
                'binary': 'binary',
                'host': 'host',
                'limit': 'limit',
                'marker': 'marker',
                'name': 'binary',
            },
            sot._query_mapping._mapping,
        )

    def test_make_it(self):
        sot = service.Service(**EXAMPLE)
        self.assertEqual(EXAMPLE['host'], sot.host)
        self.assertEqual(EXAMPLE['binary'], sot.binary)
        self.assertEqual(EXAMPLE['binary'], sot.name)
        self.assertEqual(EXAMPLE['status'], sot.status)
        self.assertEqual(EXAMPLE['state'], sot.state)
        self.assertEqual(EXAMPLE['zone'], sot.availability_zone)
        self.assertEqual(EXAMPLE['id'], sot.id)

    def test_find_single_match(self):
        data = [
            service.Service(name='bin1', host='host', id=1),
            service.Service(name='bin2', host='host', id=2),
        ]
        with mock.patch.object(service.Service, 'list') as list_mock:
            list_mock.return_value = data

            sot = service.Service.find(
                self.sess, 'bin1', ignore_missing=True, host='host'
            )

            self.assertEqual(data[0], sot)

    def test_find_with_id_single_match(self):
        data = [
            service.Service(name='bin1', host='host', id=1),
            service.Service(name='bin2', host='host', id='2'),
        ]
        with mock.patch.object(service.Service, 'list') as list_mock:
            list_mock.return_value = data

            sot = service.Service.find(
                self.sess, '2', ignore_missing=True, binary='bin1', host='host'
            )

            self.assertEqual(data[1], sot)

            # Verify find when ID is int
            sot = service.Service.find(
                self.sess, 1, ignore_missing=True, binary='bin1', host='host'
            )

            self.assertEqual(data[0], sot)

    def test_find_no_match(self):
        data = [
            service.Service(name='bin1', host='host', id=1),
            service.Service(name='bin2', host='host', id=2),
        ]
        with mock.patch.object(service.Service, 'list') as list_mock:
            list_mock.return_value = data

            self.assertIsNone(
                service.Service.find(
                    self.sess, 'fake', ignore_missing=True, host='host'
                )
            )

    def test_find_no_match_exception(self):
        data = [
            service.Service(name='bin1', host='host', id=1),
            service.Service(name='bin2', host='host', id=2),
        ]
        with mock.patch.object(service.Service, 'list') as list_mock:
            list_mock.return_value = data

            self.assertRaises(
                exceptions.ResourceNotFound,
                service.Service.find,
                self.sess,
                'fake',
                ignore_missing=False,
                host='host',
            )

    def test_find_multiple_match(self):
        data = [
            service.Service(name='bin1', host='host', id=1),
            service.Service(name='bin1', host='host', id=2),
        ]
        with mock.patch.object(service.Service, 'list') as list_mock:
            list_mock.return_value = data

            self.assertRaises(
                exceptions.DuplicateResource,
                service.Service.find,
                self.sess,
                'bin1',
                ignore_missing=False,
                host='host',
            )

    @mock.patch(
        'openstack.utils.supports_microversion',
        autospec=True,
        return_value=False,
    )
    def test_set_forced_down_before_211(self, mv_mock):
        sot = service.Service(**EXAMPLE)

        res = sot.set_forced_down(self.sess, 'host1', 'nova-compute', True)
        self.assertIsNotNone(res)

        url = 'os-services/force-down'
        body = {
            'binary': 'nova-compute',
            'host': 'host1',
        }
        self.sess.put.assert_called_with(
            url, json=body, microversion=self.sess.default_microversion
        )

    @mock.patch(
        'openstack.utils.supports_microversion',
        autospec=True,
        return_value=True,
    )
    def test_set_forced_down_after_211(self, mv_mock):
        sot = service.Service(**EXAMPLE)

        res = sot.set_forced_down(self.sess, 'host1', 'nova-compute', True)
        self.assertIsNotNone(res)

        url = 'os-services/force-down'
        body = {
            'binary': 'nova-compute',
            'host': 'host1',
            'forced_down': True,
        }
        self.sess.put.assert_called_with(url, json=body, microversion='2.11')

    @mock.patch(
        'openstack.utils.supports_microversion',
        autospec=True,
        return_value=True,
    )
    def test_set_forced_down_after_253(self, mv_mock):
        sot = service.Service(**EXAMPLE)

        res = sot.set_forced_down(self.sess, None, None, True)
        self.assertIsNotNone(res)

        url = 'os-services/force-down'
        body = {
            'binary': sot.binary,
            'host': sot.host,
            'forced_down': True,
        }
        self.sess.put.assert_called_with(url, json=body, microversion='2.11')

    def test_enable(self):
        sot = service.Service(**EXAMPLE)

        res = sot.enable(self.sess, 'host1', 'nova-compute')
        self.assertIsNotNone(res)

        url = 'os-services/enable'
        body = {
            'binary': 'nova-compute',
            'host': 'host1',
        }
        self.sess.put.assert_called_with(
            url, json=body, microversion=self.sess.default_microversion
        )

    def test_disable(self):
        sot = service.Service(**EXAMPLE)

        res = sot.disable(self.sess, 'host1', 'nova-compute')
        self.assertIsNotNone(res)

        url = 'os-services/disable'
        body = {
            'binary': 'nova-compute',
            'host': 'host1',
        }
        self.sess.put.assert_called_with(
            url, json=body, microversion=self.sess.default_microversion
        )

    def test_disable_with_reason(self):
        sot = service.Service(**EXAMPLE)
        reason = 'fencing'

        res = sot.disable(self.sess, 'host1', 'nova-compute', reason=reason)

        self.assertIsNotNone(res)

        url = 'os-services/disable-log-reason'
        body = {
            'binary': 'nova-compute',
            'host': 'host1',
            'disabled_reason': reason,
        }
        self.sess.put.assert_called_with(
            url, json=body, microversion=self.sess.default_microversion
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from keystoneauth1 import adapter

from openstack.dns.v2 import zone_share
from openstack.tests.unit import base


class TestZoneShare(base.TestCase):
    def setUp(self):
        super(TestZoneShare, self).setUp()
        self.resp = mock.Mock()
        self.resp.body = None
        self.resp.json = mock.Mock(return_value=self.resp.body)
        self.resp.status_code = 200
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.post = mock.Mock(return_value=self.resp)
        self.sess.default_microversion = None

    def test_basic(self):
        sot = zone_share.ZoneShare()
        self.assertEqual(None, sot.resource_key)
        self.assertEqual('shared_zones', sot.resources_key)
        self.assertEqual('/zones/%(zone_id)s/shares', sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_delete)
        self.assertFalse(sot.allow_commit)

        self.assertDictEqual(
            {
                'target_project_id': 'target_project_id',
                'limit': 'limit',
                'marker': 'marker',
            },
            sot._query_mapping._mapping,
        )

    def test_make_it(self):
        share_id = 'bogus_id'
        zone_id = 'bogus_zone_id'
        project_id = 'bogus_project_id'
        target_id = 'bogus_target_id'
        expected = {
            'id': share_id,
            'zone_id': zone_id,
            'project_id': project_id,
            'target_project_id': target_id,
        }

        sot = zone_share.ZoneShare(**expected)
        self.assertEqual(share_id, sot.id)
        self.assertEqual(zone_id, sot.zone_id)
        self.assertEqual(project_id, sot.project_id)
        self.assertEqual(target_id, sot.target_project_id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class BlockStorageSummary(resource.Resource):
    base_path = "/volumes/summary"

    # capabilities
    allow_fetch = True

    # Properties
    #: Total size of all the volumes
    total_size = resource.Body("total_size")
    #: Total count of all the volumes
    total_count = resource.Body("total_count")
    #: Metadata of all the volumes
    metadata = resource.Body("metadata")

    _max_microversion = "3.36"



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class PortForwarding(resource.Resource):
    name_attribute = "floating_ip_port_forwarding"
    resource_name = "port forwarding"
    resource_key = 'port_forwarding'
    resources_key = 'port_forwardings'
    base_path = '/floatingips/%(floatingip_id)s/port_forwardings'

    _allow_unknown_attrs_in_body = True

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'internal_port_id',
        'external_port',
        'protocol',
    )

    # Properties
    #: The ID of Floating IP address
    floatingip_id = resource.URI('floatingip_id')
    #: The ID of internal port
    internal_port_id = resource.Body('internal_port_id')
    #: The internal IP address
    internal_ip_address = resource.Body('internal_ip_address')
    #: The internal TCP/UDP/other port number
    internal_port = resource.Body('internal_port', type=int)
    #: The external TCP/UDP/other port number
    external_port = resource.Body('external_port', type=int)
    #: The protocol
    protocol = resource.Body('protocol')
    #: The description
    description = resource.Body('description')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from openstack import exceptions
from openstack.tests.functional.baremetal import base


class TestBareMetalChassis(base.BaseBaremetalTest):
    def test_chassis_create_get_delete(self):
        chassis = self.create_chassis()

        loaded = self.conn.baremetal.get_chassis(chassis.id)
        self.assertEqual(loaded.id, chassis.id)

        self.conn.baremetal.delete_chassis(chassis, ignore_missing=False)
        self.assertRaises(
            exceptions.ResourceNotFound,
            self.conn.baremetal.get_chassis,
            chassis.id,
        )

    def test_chassis_update(self):
        chassis = self.create_chassis()
        chassis.extra = {'answer': 42}

        chassis = self.conn.baremetal.update_chassis(chassis)
        self.assertEqual({'answer': 42}, chassis.extra)

        chassis = self.conn.baremetal.get_chassis(chassis.id)
        self.assertEqual({'answer': 42}, chassis.extra)

    def test_chassis_patch(self):
        chassis = self.create_chassis()

        chassis = self.conn.baremetal.patch_chassis(
            chassis, dict(path='/extra/answer', op='add', value=42)
        )
        self.assertEqual({'answer': 42}, chassis.extra)

        chassis = self.conn.baremetal.get_chassis(chassis.id)
        self.assertEqual({'answer': 42}, chassis.extra)

    def test_chassis_negative_non_existing(self):
        uuid = "5c9dcd04-2073-49bc-9618-99ae634d8971"
        self.assertRaises(
            exceptions.ResourceNotFound, self.conn.baremetal.get_chassis, uuid
        )
        self.assertRaises(
            exceptions.ResourceNotFound,
            self.conn.baremetal.find_chassis,
            uuid,
            ignore_missing=False,
        )
        self.assertRaises(
            exceptions.ResourceNotFound,
            self.conn.baremetal.delete_chassis,
            uuid,
            ignore_missing=False,
        )
        self.assertIsNone(self.conn.baremetal.find_chassis(uuid))
        self.assertIsNone(self.conn.baremetal.delete_chassis(uuid))


class TestBareMetalChassisFields(base.BaseBaremetalTest):
    min_microversion = '1.8'

    def test_chassis_fields(self):
        self.create_chassis(description='something')
        result = self.conn.baremetal.chassis(fields=['uuid', 'extra'])
        for ch in result:
            self.assertIsNotNone(ch.id)
            self.assertIsNone(ch.description)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.image.v2 import metadef_namespace as _metadef_namespace
from openstack.image.v2 import metadef_object as _metadef_object
from openstack.tests.functional.image.v2 import base


class TestMetadefObject(base.BaseImageTest):
    def setUp(self):
        super().setUp()

        # create namespace for object
        namespace = self.getUniqueString().split('.')[-1]
        self.metadef_namespace = self.conn.image.create_metadef_namespace(
            namespace=namespace,
        )
        self.assertIsInstance(
            self.metadef_namespace,
            _metadef_namespace.MetadefNamespace,
        )
        self.assertEqual(namespace, self.metadef_namespace.namespace)

        # create object
        object = self.getUniqueString().split('.')[-1]
        self.metadef_object = self.conn.image.create_metadef_object(
            name=object,
            namespace=self.metadef_namespace,
        )
        self.assertIsInstance(
            self.metadef_object,
            _metadef_object.MetadefObject,
        )
        self.assertEqual(object, self.metadef_object.name)

    def tearDown(self):
        self.conn.image.delete_metadef_object(
            self.metadef_object,
            self.metadef_object.namespace_name,
        )
        self.conn.image.wait_for_delete(self.metadef_object)

        self.conn.image.delete_metadef_namespace(self.metadef_namespace)
        self.conn.image.wait_for_delete(self.metadef_namespace)

        super().tearDown()

    def test_metadef_objects(self):
        # get
        metadef_object = self.conn.image.get_metadef_object(
            self.metadef_object.name,
            self.metadef_namespace,
        )
        self.assertEqual(
            self.metadef_object.namespace_name,
            metadef_object.namespace_name,
        )
        self.assertEqual(
            self.metadef_object.name,
            metadef_object.name,
        )

        # list
        metadef_objects = list(
            self.conn.image.metadef_objects(self.metadef_object.namespace_name)
        )
        # there are a load of default metadef objects so we don't assert
        # that this is the *only* metadef objects present
        self.assertIn(
            self.metadef_object.name,
            {o.name for o in metadef_objects},
        )

        # update
        metadef_object_new_name = 'New object name'
        metadef_object_new_description = 'New object description'
        metadef_object = self.conn.image.update_metadef_object(
            self.metadef_object.name,
            namespace=self.metadef_object.namespace_name,
            name=metadef_object_new_name,
            description=metadef_object_new_description,
        )
        self.assertIsInstance(
            metadef_object,
            _metadef_object.MetadefObject,
        )
        self.assertEqual(
            metadef_object_new_name,
            metadef_object.name,
        )
        self.assertEqual(
            metadef_object_new_description,
            metadef_object.description,
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from openstack import exceptions
from openstack.tests.functional.baremetal import base


class TestBareMetalVolumetarget(base.BaseBaremetalTest):
    min_microversion = '1.32'

    def setUp(self):
        super(TestBareMetalVolumetarget, self).setUp()
        self.node = self.create_node(provision_state='enroll')

    def test_volume_target_create_get_delete(self):
        self.conn.baremetal.set_node_provision_state(
            self.node, 'manage', wait=True
        )
        self.conn.baremetal.set_node_power_state(self.node, 'power off')
        volume_target = self.create_volume_target(
            boot_index=0,
            volume_id='04452bed-5367-4202-8bf5-de4335ac56d2',
            volume_type='iscsi',
        )

        loaded = self.conn.baremetal.get_volume_target(volume_target.id)
        self.assertEqual(loaded.id, volume_target.id)
        self.assertIsNotNone(loaded.node_id)

        with_fields = self.conn.baremetal.get_volume_target(
            volume_target.id, fields=['uuid', 'extra']
        )
        self.assertEqual(volume_target.id, with_fields.id)
        self.assertIsNone(with_fields.node_id)

        self.conn.baremetal.delete_volume_target(
            volume_target, ignore_missing=False
        )
        self.assertRaises(
            exceptions.ResourceNotFound,
            self.conn.baremetal.get_volume_target,
            volume_target.id,
        )

    def test_volume_target_list(self):
        node2 = self.create_node(name='test-node')
        self.conn.baremetal.set_node_provision_state(
            node2, 'manage', wait=True
        )
        self.conn.baremetal.set_node_power_state(node2, 'power off')
        self.conn.baremetal.set_node_provision_state(
            self.node, 'manage', wait=True
        )
        self.conn.baremetal.set_node_power_state(self.node, 'power off')
        vt1 = self.create_volume_target(
            boot_index=0,
            volume_id='bd4d008c-7d31-463d-abf9-6c23d9d55f7f',
            node_id=node2.id,
            volume_type='iscsi',
        )
        vt2 = self.create_volume_target(
            boot_index=0,
            volume_id='04452bed-5367-4202-8bf5-de4335ac57c2',
            node_id=self.node.id,
            volume_type='iscsi',
        )

        vts = self.conn.baremetal.volume_targets(node=self.node.id)
        self.assertEqual([v.id for v in vts], [vt2.id])

        vts = self.conn.baremetal.volume_targets(node=node2.id)
        self.assertEqual([v.id for v in vts], [vt1.id])

        vts = self.conn.baremetal.volume_targets(node='test-node')
        self.assertEqual([v.id for v in vts], [vt1.id])

        vts_with_details = self.conn.baremetal.volume_targets(details=True)
        for i in vts_with_details:
            self.assertIsNotNone(i.id)
            self.assertIsNotNone(i.volume_type)

        vts_with_fields = self.conn.baremetal.volume_targets(
            fields=['uuid', 'node_uuid']
        )
        for i in vts_with_fields:
            self.assertIsNotNone(i.id)
            self.assertIsNone(i.volume_type)
            self.assertIsNotNone(i.node_id)

    def test_volume_target_list_update_delete(self):
        self.conn.baremetal.set_node_provision_state(
            self.node, 'manage', wait=True
        )
        self.conn.baremetal.set_node_power_state(self.node, 'power off')
        self.create_volume_target(
            boot_index=0,
            volume_id='04452bed-5367-4202-8bf5-de4335ac57h3',
            node_id=self.node.id,
            volume_type='iscsi',
            extra={'foo': 'bar'},
        )
        volume_target = next(
            self.conn.baremetal.volume_targets(details=True, node=self.node.id)
        )
        self.assertEqual(volume_target.extra, {'foo': 'bar'})

        # This test checks that resources returned from listing are usable
        self.conn.baremetal.update_volume_target(
            volume_target, extra={'foo': 42}
        )
        self.conn.baremetal.delete_volume_target(
            volume_target, ignore_missing=False
        )

    def test_volume_target_update(self):
        self.conn.baremetal.set_node_provision_state(
            self.node, 'manage', wait=True
        )
        self.conn.baremetal.set_node_power_state(self.node, 'power off')
        volume_target = self.create_volume_target(
            boot_index=0,
            volume_id='04452bed-5367-4202-8bf5-de4335ac53h7',
            node_id=self.node.id,
            volume_type='isci',
        )
        volume_target.extra = {'answer': 42}

        volume_target = self.conn.baremetal.update_volume_target(volume_target)
        self.assertEqual({'answer': 42}, volume_target.extra)

        volume_target = self.conn.baremetal.get_volume_target(volume_target.id)
        self.assertEqual({'answer': 42}, volume_target.extra)

    def test_volume_target_patch(self):
        vol_targ_id = '04452bed-5367-4202-9cg6-de4335ac53h7'
        self.conn.baremetal.set_node_provision_state(
            self.node, 'manage', wait=True
        )
        self.conn.baremetal.set_node_power_state(self.node, 'power off')
        volume_target = self.create_volume_target(
            boot_index=0,
            volume_id=vol_targ_id,
            node_id=self.node.id,
            volume_type='isci',
        )

        volume_target = self.conn.baremetal.patch_volume_target(
            volume_target, dict(path='/extra/answer', op='add', value=42)
        )
        self.assertEqual({'answer': 42}, volume_target.extra)
        self.assertEqual(vol_targ_id, volume_target.volume_id)

        volume_target = self.conn.baremetal.get_volume_target(volume_target.id)
        self.assertEqual({'answer': 42}, volume_target.extra)

    def test_volume_target_negative_non_existing(self):
        uuid = "5c9dcd04-2073-49bc-9618-99ae634d8971"
        self.assertRaises(
            exceptions.ResourceNotFound,
            self.conn.baremetal.get_volume_target,
            uuid,
        )
        self.assertRaises(
            exceptions.ResourceNotFound,
            self.conn.baremetal.find_volume_target,
            uuid,
            ignore_missing=False,
        )
        self.assertRaises(
            exceptions.ResourceNotFound,
            self.conn.baremetal.delete_volume_target,
            uuid,
            ignore_missing=False,
        )
        self.assertIsNone(self.conn.baremetal.find_volume_target(uuid))
        self.assertIsNone(self.conn.baremetal.delete_volume_target(uuid))

    def test_volume_target_fields(self):
        self.create_node()
        self.conn.baremetal.set_node_provision_state(
            self.node, 'manage', wait=True
        )
        self.conn.baremetal.set_node_power_state(self.node, 'power off')
        self.create_volume_target(
            boot_index=0,
            volume_id='04452bed-5367-4202-8bf5-99ae634d8971',
            node_id=self.node.id,
            volume_type='iscsi',
        )
        result = self.conn.baremetal.volume_targets(fields=['uuid', 'node_id'])
        for item in result:
            self.assertIsNotNone(item.id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.identity.v2 import _proxy
from openstack.identity.v2 import role
from openstack.identity.v2 import tenant
from openstack.identity.v2 import user
from openstack.tests.unit import test_proxy_base as test_proxy_base


class TestIdentityProxy(test_proxy_base.TestProxyBase):
    def setUp(self):
        super(TestIdentityProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)

    def test_role_create_attrs(self):
        self.verify_create(self.proxy.create_role, role.Role)

    def test_role_delete(self):
        self.verify_delete(self.proxy.delete_role, role.Role, False)

    def test_role_delete_ignore(self):
        self.verify_delete(self.proxy.delete_role, role.Role, True)

    def test_role_find(self):
        self.verify_find(self.proxy.find_role, role.Role)

    def test_role_get(self):
        self.verify_get(self.proxy.get_role, role.Role)

    def test_roles(self):
        self.verify_list(self.proxy.roles, role.Role)

    def test_role_update(self):
        self.verify_update(self.proxy.update_role, role.Role)

    def test_tenant_create_attrs(self):
        self.verify_create(self.proxy.create_tenant, tenant.Tenant)

    def test_tenant_delete(self):
        self.verify_delete(self.proxy.delete_tenant, tenant.Tenant, False)

    def test_tenant_delete_ignore(self):
        self.verify_delete(self.proxy.delete_tenant, tenant.Tenant, True)

    def test_tenant_find(self):
        self.verify_find(self.proxy.find_tenant, tenant.Tenant)

    def test_tenant_get(self):
        self.verify_get(self.proxy.get_tenant, tenant.Tenant)

    def test_tenants(self):
        self.verify_list(self.proxy.tenants, tenant.Tenant)

    def test_tenant_update(self):
        self.verify_update(self.proxy.update_tenant, tenant.Tenant)

    def test_user_create_attrs(self):
        self.verify_create(self.proxy.create_user, user.User)

    def test_user_delete(self):
        self.verify_delete(self.proxy.delete_user, user.User, False)

    def test_user_delete_ignore(self):
        self.verify_delete(self.proxy.delete_user, user.User, True)

    def test_user_find(self):
        self.verify_find(self.proxy.find_user, user.User)

    def test_user_get(self):
        self.verify_get(self.proxy.get_user, user.User)

    def test_users(self):
        self.verify_list(self.proxy.users, user.User)

    def test_user_update(self):
        self.verify_update(self.proxy.update_user, user.User)



# Copyright 2020 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os
import tempfile
import textwrap

from openstack.config import loader
from openstack import exceptions
from openstack.tests.unit.config import base

FILES = {
    'yaml': textwrap.dedent(
        '''
        foo: bar
        baz:
            - 1
            - 2
            - 3
    '''
    ),
    'json': textwrap.dedent(
        '''
        {
            "foo": "bar",
            "baz": [
                1,
                2,
                3
            ]
        }
    '''
    ),
    'txt': textwrap.dedent(
        '''
        foo
        bar baz
        test
            one two
    '''
    ),
}


class TestLoader(base.TestCase):
    def test_base_load_yaml_json_file(self):
        with tempfile.TemporaryDirectory() as tmpdir:
            tested_files = []
            for key, value in FILES.items():
                fn = os.path.join(tmpdir, 'file.{ext}'.format(ext=key))
                with open(fn, 'w+') as fp:
                    fp.write(value)
                tested_files.append(fn)

            path, result = loader.OpenStackConfig()._load_yaml_json_file(
                tested_files
            )
            # NOTE(hberaud): Prefer to test path rather than file because
            # our FILES var is a dict so results are appened
            # without keeping the initial order (python 3.5)
            self.assertEqual(tmpdir, os.path.dirname(path))

    def test__load_yaml_json_file_without_json(self):
        with tempfile.TemporaryDirectory() as tmpdir:
            tested_files = []
            for key, value in FILES.items():
                if key == 'json':
                    continue
                fn = os.path.join(tmpdir, 'file.{ext}'.format(ext=key))
                with open(fn, 'w+') as fp:
                    fp.write(value)
                tested_files.append(fn)

            path, result = loader.OpenStackConfig()._load_yaml_json_file(
                tested_files
            )
            # NOTE(hberaud): Prefer to test path rather than file because
            # our FILES var is a dict so results are appened
            # without keeping the initial order (python 3.5)
            self.assertEqual(tmpdir, os.path.dirname(path))

    def test__load_yaml_json_file_without_json_yaml(self):
        with tempfile.TemporaryDirectory() as tmpdir:
            tested_files = []
            fn = os.path.join(tmpdir, 'file.txt')
            with open(fn, 'w+') as fp:
                fp.write(FILES['txt'])
            tested_files.append(fn)

            path, result = loader.OpenStackConfig()._load_yaml_json_file(
                tested_files
            )
            self.assertEqual(fn, path)

    def test__load_yaml_json_file_without_perm(self):
        with tempfile.TemporaryDirectory() as tmpdir:
            tested_files = []
            fn = os.path.join(tmpdir, 'file.txt')
            with open(fn, 'w+') as fp:
                fp.write(FILES['txt'])
            os.chmod(fn, 222)
            tested_files.append(fn)

            path, result = loader.OpenStackConfig()._load_yaml_json_file(
                tested_files
            )
            self.assertEqual(None, path)

    def test__load_yaml_json_file_nonexisting(self):
        tested_files = []
        fn = os.path.join('/fake', 'file.txt')
        tested_files.append(fn)

        path, result = loader.OpenStackConfig()._load_yaml_json_file(
            tested_files
        )
        self.assertEqual(None, path)


class TestFixArgv(base.TestCase):
    def test_no_changes(self):
        argv = [
            '-a',
            '-b',
            '--long-arg',
            '--multi-value',
            'key1=value1',
            '--multi-value',
            'key2=value2',
        ]
        expected = argv[:]
        loader._fix_argv(argv)
        self.assertEqual(expected, argv)

    def test_replace(self):
        argv = [
            '-a',
            '-b',
            '--long-arg',
            '--multi_value',
            'key1=value1',
            '--multi_value',
            'key2=value2',
        ]
        expected = [
            '-a',
            '-b',
            '--long-arg',
            '--multi-value',
            'key1=value1',
            '--multi-value',
            'key2=value2',
        ]
        loader._fix_argv(argv)
        self.assertEqual(expected, argv)

    def test_mix(self):
        argv = [
            '-a',
            '-b',
            '--long-arg',
            '--multi_value',
            'key1=value1',
            '--multi-value',
            'key2=value2',
        ]
        self.assertRaises(exceptions.ConfigException, loader._fix_argv, argv)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid

from openstack import exceptions
from openstack.tests import fakes
from openstack.tests.unit import base


class TestNovaExtensions(base.TestCase):
    def test__nova_extensions(self):
        body = [
            {
                "updated": "2014-12-03T00:00:00Z",
                "name": "Multinic",
                "links": [],
                "namespace": "http://openstack.org/compute/ext/fake_xml",
                "alias": "NMN",
                "description": "Multiple network support.",
            },
            {
                "updated": "2014-12-03T00:00:00Z",
                "name": "DiskConfig",
                "links": [],
                "namespace": "http://openstack.org/compute/ext/fake_xml",
                "alias": "OS-DCF",
                "description": "Disk Management Extension.",
            },
        ]
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri='{endpoint}/extensions'.format(
                        endpoint=fakes.COMPUTE_ENDPOINT
                    ),
                    json=dict(extensions=body),
                )
            ]
        )
        extensions = self.cloud._nova_extensions()
        self.assertEqual(set(['NMN', 'OS-DCF']), extensions)

        self.assert_calls()

    def test__nova_extensions_fails(self):
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri='{endpoint}/extensions'.format(
                        endpoint=fakes.COMPUTE_ENDPOINT
                    ),
                    status_code=404,
                ),
            ]
        )
        self.assertRaises(
            exceptions.ResourceNotFound, self.cloud._nova_extensions
        )

        self.assert_calls()

    def test__has_nova_extension(self):
        body = [
            {
                "updated": "2014-12-03T00:00:00Z",
                "name": "Multinic",
                "links": [],
                "namespace": "http://openstack.org/compute/ext/fake_xml",
                "alias": "NMN",
                "description": "Multiple network support.",
            },
            {
                "updated": "2014-12-03T00:00:00Z",
                "name": "DiskConfig",
                "links": [],
                "namespace": "http://openstack.org/compute/ext/fake_xml",
                "alias": "OS-DCF",
                "description": "Disk Management Extension.",
            },
        ]
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri='{endpoint}/extensions'.format(
                        endpoint=fakes.COMPUTE_ENDPOINT
                    ),
                    json=dict(extensions=body),
                )
            ]
        )
        self.assertTrue(self.cloud._has_nova_extension('NMN'))

        self.assert_calls()

    def test__has_nova_extension_missing(self):
        body = [
            {
                "updated": "2014-12-03T00:00:00Z",
                "name": "Multinic",
                "links": [],
                "namespace": "http://openstack.org/compute/ext/fake_xml",
                "alias": "NMN",
                "description": "Multiple network support.",
            },
            {
                "updated": "2014-12-03T00:00:00Z",
                "name": "DiskConfig",
                "links": [],
                "namespace": "http://openstack.org/compute/ext/fake_xml",
                "alias": "OS-DCF",
                "description": "Disk Management Extension.",
            },
        ]
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri='{endpoint}/extensions'.format(
                        endpoint=fakes.COMPUTE_ENDPOINT
                    ),
                    json=dict(extensions=body),
                )
            ]
        )
        self.assertFalse(self.cloud._has_nova_extension('invalid'))

        self.assert_calls()


class TestServers(base.TestCase):
    def test_get_server(self):
        server1 = fakes.make_fake_server('123', 'mickey')
        server2 = fakes.make_fake_server('345', 'mouse')

        self.register_uris(
            [
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', 'detail']
                    ),
                    json={'servers': [server1, server2]},
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'networks']
                    ),
                    json={"networks": []},
                ),
            ]
        )

        r = self.cloud.get_server('mickey')
        self.assertIsNotNone(r)
        self.assertEqual(server1['name'], r['name'])

        self.assert_calls()

    def test_get_server_not_found(self):
        self.register_uris(
            [
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', 'detail']
                    ),
                    json={'servers': []},
                ),
            ]
        )

        r = self.cloud.get_server('doesNotExist')
        self.assertIsNone(r)

        self.assert_calls()

    def test_list_servers(self):
        server_id = str(uuid.uuid4())
        server_name = self.getUniqueString('name')
        fake_server = fakes.make_fake_server(server_id, server_name)
        self.register_uris(
            [
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', 'detail']
                    ),
                    json={'servers': [fake_server]},
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'networks']
                    ),
                    json={"networks": []},
                ),
            ]
        )

        r = self.cloud.list_servers()

        self.assertEqual(1, len(r))
        self.assertEqual(server_name, r[0]['name'])

        self.assert_calls()

    def test_list_server_private_ip(self):
        self.has_neutron = True
        fake_server = {
            "OS-EXT-STS:task_state": None,
            "addresses": {
                "private": [
                    {
                        "OS-EXT-IPS-MAC:mac_addr": "fa:16:3e:b4:a3:07",
                        "version": 4,
                        "addr": "10.4.0.13",
                        "OS-EXT-IPS:type": "fixed",
                    },
                    {
                        "OS-EXT-IPS-MAC:mac_addr": "fa:16:3e:b4:a3:07",
                        "version": 4,
                        "addr": "89.40.216.229",
                        "OS-EXT-IPS:type": "floating",
                    },
                ]
            },
            "links": [
                {"href": "http://example.com/images/95e4c4", "rel": "self"},
                {
                    "href": "http://example.com/images/95e4c4",
                    "rel": "bookmark",
                },
            ],
            "image": {
                "id": "95e4c449-8abf-486e-97d9-dc3f82417d2d",
                "links": [
                    {
                        "href": "http://example.com/images/95e4c4",
                        "rel": "bookmark",
                    }
                ],
            },
            "OS-EXT-STS:vm_state": "active",
            "OS-SRV-USG:launched_at": "2018-03-01T02:44:50.000000",
            "flavor": {
                "id": "3bd99062-2fe8-4eac-93f0-9200cc0f97ae",
                "links": [
                    {
                        "href": "http://example.com/flavors/95e4c4",
                        "rel": "bookmark",
                    }
                ],
            },
            "id": "97fe35e9-756a-41a2-960a-1d057d2c9ee4",
            "security_groups": [{"name": "default"}],
            "user_id": "c17534835f8f42bf98fc367e0bf35e09",
            "OS-DCF:diskConfig": "MANUAL",
            "accessIPv4": "",
            "accessIPv6": "",
            "progress": 0,
            "OS-EXT-STS:power_state": 1,
            "OS-EXT-AZ:availability_zone": "nova",
            "metadata": {},
            "status": "ACTIVE",
            "updated": "2018-03-01T02:44:51Z",
            "hostId": "",
            "OS-SRV-USG:terminated_at": None,
            "key_name": None,
            "name": "mttest",
            "created": "2018-03-01T02:44:46Z",
            "tenant_id": "65222a4d09ea4c68934fa1028c77f394",
            "os-extended-volumes:volumes_attached": [],
            "config_drive": "",
        }
        fake_networks = {
            "networks": [
                {
                    "status": "ACTIVE",
                    "router:external": True,
                    "availability_zone_hints": [],
                    "availability_zones": ["nova"],
                    "description": None,
                    "subnets": [
                        "df3e17fa-a4b2-47ae-9015-bc93eb076ba2",
                        "6b0c3dc9-b0b8-4d87-976a-7f2ebf13e7ec",
                        "fc541f48-fc7f-48c0-a063-18de6ee7bdd7",
                    ],
                    "shared": False,
                    "tenant_id": "a564613210ee43708b8a7fc6274ebd63",
                    "tags": [],
                    "ipv6_address_scope": "9f03124f-89af-483a-b6fd-10f08079db4d",  # noqa: E501
                    "mtu": 1550,
                    "is_default": False,
                    "admin_state_up": True,
                    "revision_number": 0,
                    "ipv4_address_scope": None,
                    "port_security_enabled": True,
                    "project_id": "a564613210ee43708b8a7fc6274ebd63",
                    "id": "0232c17f-2096-49bc-b205-d3dcd9a30ebf",
                    "name": "ext-net",
                },
                {
                    "status": "ACTIVE",
                    "router:external": False,
                    "availability_zone_hints": [],
                    "availability_zones": ["nova"],
                    "description": "",
                    "subnets": ["f0ad1df5-53ee-473f-b86b-3604ea5591e9"],
                    "shared": False,
                    "tenant_id": "65222a4d09ea4c68934fa1028c77f394",
                    "created_at": "2016-10-22T13:46:26Z",
                    "tags": [],
                    "ipv6_address_scope": None,
                    "updated_at": "2016-10-22T13:46:26Z",
                    "admin_state_up": True,
                    "mtu": 1500,
                    "revision_number": 0,
                    "ipv4_address_scope": None,
                    "port_security_enabled": True,
                    "project_id": "65222a4d09ea4c68934fa1028c77f394",
                    "id": "2c9adcb5-c123-4c5a-a2ba-1ad4c4e1481f",
                    "name": "private",
                },
            ]
        }
        fake_subnets = {
            "subnets": [
                {
                    "service_types": [],
                    "description": "",
                    "enable_dhcp": True,
                    "tags": [],
                    "network_id": "827c6bb6-492f-4168-9577-f3a131eb29e8",
                    "tenant_id": "65222a4d09ea4c68934fa1028c77f394",
                    "created_at": "2017-06-12T13:23:57Z",
                    "dns_nameservers": [],
                    "updated_at": "2017-06-12T13:23:57Z",
                    "gateway_ip": "10.24.4.1",
                    "ipv6_ra_mode": None,
                    "allocation_pools": [
                        {"start": "10.24.4.2", "end": "10.24.4.254"}
                    ],
                    "host_routes": [],
                    "revision_number": 0,
                    "ip_version": 4,
                    "ipv6_address_mode": None,
                    "cidr": "10.24.4.0/24",
                    "project_id": "65222a4d09ea4c68934fa1028c77f394",
                    "id": "3f0642d9-4644-4dff-af25-bcf64f739698",
                    "subnetpool_id": None,
                    "name": "foo_subnet",
                },
                {
                    "service_types": [],
                    "description": "",
                    "enable_dhcp": True,
                    "tags": [],
                    "network_id": "2c9adcb5-c123-4c5a-a2ba-1ad4c4e1481f",
                    "tenant_id": "65222a4d09ea4c68934fa1028c77f394",
                    "created_at": "2016-10-22T13:46:26Z",
                    "dns_nameservers": ["89.36.90.101", "89.36.90.102"],
                    "updated_at": "2016-10-22T13:46:26Z",
                    "gateway_ip": "10.4.0.1",
                    "ipv6_ra_mode": None,
                    "allocation_pools": [
                        {"start": "10.4.0.2", "end": "10.4.0.200"}
                    ],
                    "host_routes": [],
                    "revision_number": 0,
                    "ip_version": 4,
                    "ipv6_address_mode": None,
                    "cidr": "10.4.0.0/24",
                    "project_id": "65222a4d09ea4c68934fa1028c77f394",
                    "id": "f0ad1df5-53ee-473f-b86b-3604ea5591e9",
                    "subnetpool_id": None,
                    "name": "private-subnet-ipv4",
                },
            ]
        }
        self.register_uris(
            [
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', 'detail']
                    ),
                    json={'servers': [fake_server]},
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'networks']
                    ),
                    json=fake_networks,
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'subnets']
                    ),
                    json=fake_subnets,
                ),
            ]
        )

        r = self.cloud.get_server('97fe35e9-756a-41a2-960a-1d057d2c9ee4')

        self.assertEqual('10.4.0.13', r['private_v4'])

        self.assert_calls()

    def test_list_servers_all_projects(self):
        """This test verifies that when list_servers is called with
        `all_projects=True` that it passes `all_tenants=True` to nova."""
        self.register_uris(
            [
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute',
                        'public',
                        append=['servers', 'detail'],
                        qs_elements=['all_tenants=True'],
                    ),
                    complete_qs=True,
                    json={'servers': []},
                ),
            ]
        )

        self.cloud.list_servers(all_projects=True)

        self.assert_calls()

    def test_list_servers_filters(self):
        """This test verifies that when list_servers is called with
        `filters` dict that it passes it to nova."""
        self.register_uris(
            [
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute',
                        'public',
                        append=['servers', 'detail'],
                        qs_elements=[
                            'deleted=True',
                            'changes-since=2014-12-03T00:00:00Z',
                        ],
                    ),
                    complete_qs=True,
                    json={'servers': []},
                ),
            ]
        )

        self.cloud.list_servers(
            filters={'deleted': True, 'changes-since': '2014-12-03T00:00:00Z'}
        )

        self.assert_calls()

    def test_list_servers_exception(self):
        self.register_uris(
            [
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', 'detail']
                    ),
                    status_code=400,
                ),
            ]
        )

        self.assertRaises(exceptions.SDKException, self.cloud.list_servers)

        self.assert_calls()



# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import functools

from openstack.cloud import _utils
from openstack.config import loader
from openstack import connection
from openstack import exceptions

__all__ = ['OpenStackInventory']


class OpenStackInventory:
    # Put this here so the capability can be detected with hasattr on the class
    extra_config = None

    def __init__(
        self,
        config_files=None,
        refresh=False,
        private=False,
        config_key=None,
        config_defaults=None,
        cloud=None,
        use_direct_get=False,
    ):
        if config_files is None:
            config_files = []
        config = loader.OpenStackConfig(
            config_files=loader.CONFIG_FILES + config_files
        )
        self.extra_config = config.get_extra_config(
            config_key, config_defaults
        )

        if cloud is None:
            self.clouds = [
                connection.Connection(config=cloud_region)
                for cloud_region in config.get_all()
            ]
        else:
            self.clouds = [connection.Connection(config=config.get_one(cloud))]

        if private:
            for cloud in self.clouds:
                cloud.private = True

        # Handle manual invalidation of entire persistent cache
        if refresh:
            for cloud in self.clouds:
                cloud._cache.invalidate()

    def list_hosts(
        self, expand=True, fail_on_cloud_config=True, all_projects=False
    ):
        hostvars = []

        for cloud in self.clouds:
            try:
                # Cycle on servers
                for server in cloud.list_servers(
                    detailed=expand, all_projects=all_projects
                ):
                    hostvars.append(server)
            except exceptions.SDKException:
                # Don't fail on one particular cloud as others may work
                if fail_on_cloud_config:
                    raise

        return hostvars

    def search_hosts(self, name_or_id=None, filters=None, expand=True):
        hosts = self.list_hosts(expand=expand)
        return _utils._filter_list(hosts, name_or_id, filters)

    def get_host(self, name_or_id, filters=None, expand=True):
        if expand:
            func = self.search_hosts
        else:
            func = functools.partial(self.search_hosts, expand=False)
        return _utils._get_entity(self, func, name_or_id, filters)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from openstack.container_infrastructure_management.v1 import (
    cluster_certificate,
)
from openstack.tests.unit import base

coe_cluster_ca_obj = dict(
    cluster_uuid="43e305ce-3a5f-412a-8a14-087834c34c8c",
    pem="-----BEGIN CERTIFICATE-----\nMIIDAO\n-----END CERTIFICATE-----\n",
    bay_uuid="43e305ce-3a5f-412a-8a14-087834c34c8c",
    links=[],
)

coe_cluster_signed_cert_obj = dict(
    cluster_uuid='43e305ce-3a5f-412a-8a14-087834c34c8c',
    pem='-----BEGIN CERTIFICATE-----\nMIIDAO\n-----END CERTIFICATE-----',
    bay_uuid='43e305ce-3a5f-412a-8a14-087834c34c8c',
    links=[],
    csr=(
        '-----BEGIN CERTIFICATE REQUEST-----\nMIICfz=='
        '\n-----END CERTIFICATE REQUEST-----\n'
    ),
)


class TestCOEClusters(base.TestCase):
    def _compare_cluster_certs(self, exp, real):
        self.assertDictEqual(
            cluster_certificate.ClusterCertificate(**exp).to_dict(
                computed=False
            ),
            real.to_dict(computed=False),
        )

    def get_mock_url(
        self,
        service_type='container-infrastructure-management',
        base_url_append=None,
        append=None,
        resource=None,
    ):
        return super(TestCOEClusters, self).get_mock_url(
            service_type=service_type,
            resource=resource,
            append=append,
            base_url_append=base_url_append,
        )

    def test_get_coe_cluster_certificate(self):
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        resource='certificates',
                        append=[coe_cluster_ca_obj['cluster_uuid']],
                    ),
                    json=coe_cluster_ca_obj,
                )
            ]
        )
        ca_cert = self.cloud.get_coe_cluster_certificate(
            coe_cluster_ca_obj['cluster_uuid']
        )
        self._compare_cluster_certs(coe_cluster_ca_obj, ca_cert)
        self.assert_calls()

    def test_sign_coe_cluster_certificate(self):
        self.register_uris(
            [
                dict(
                    method='POST',
                    uri=self.get_mock_url(resource='certificates'),
                    json={
                        "cluster_uuid": coe_cluster_signed_cert_obj[
                            'cluster_uuid'
                        ],
                        "csr": coe_cluster_signed_cert_obj['csr'],
                    },
                )
            ]
        )
        self.cloud.sign_coe_cluster_certificate(
            coe_cluster_signed_cert_obj['cluster_uuid'],
            coe_cluster_signed_cert_obj['csr'],
        )
        self.assert_calls()



# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
test_port
----------------------------------

Test port resource (managed by neutron)
"""

from openstack import exceptions
from openstack.network.v2 import port as _port
from openstack.tests.unit import base


class TestPort(base.TestCase):
    mock_neutron_port_create_rep = {
        'port': {
            'status': 'DOWN',
            'binding:host_id': '',
            'name': 'test-port-name',
            'allowed_address_pairs': [],
            'admin_state_up': True,
            'network_id': 'test-net-id',
            'tenant_id': 'test-tenant-id',
            'binding:vif_details': {},
            'binding:vnic_type': 'normal',
            'binding:vif_type': 'unbound',
            'extra_dhcp_opts': [],
            'device_owner': '',
            'mac_address': '50:1c:0d:e4:f0:0d',
            'binding:profile': {},
            'fixed_ips': [
                {'subnet_id': 'test-subnet-id', 'ip_address': '29.29.29.29'}
            ],
            'id': 'test-port-id',
            'security_groups': [],
            'device_id': '',
        }
    }

    mock_neutron_port_update_rep = {
        'port': {
            'status': 'DOWN',
            'binding:host_id': '',
            'name': 'test-port-name-updated',
            'allowed_address_pairs': [],
            'admin_state_up': True,
            'network_id': 'test-net-id',
            'tenant_id': 'test-tenant-id',
            'binding:vif_details': {},
            'extra_dhcp_opts': [],
            'binding:vnic_type': 'normal',
            'binding:vif_type': 'unbound',
            'device_owner': '',
            'mac_address': '50:1c:0d:e4:f0:0d',
            'binding:profile': {},
            'fixed_ips': [
                {'subnet_id': 'test-subnet-id', 'ip_address': '29.29.29.29'}
            ],
            'id': 'test-port-id',
            'security_groups': [],
            'device_id': '',
        }
    }

    mock_neutron_port_list_rep = {
        'ports': [
            {
                'status': 'ACTIVE',
                'binding:host_id': 'devstack',
                'name': 'first-port',
                'allowed_address_pairs': [],
                'admin_state_up': True,
                'network_id': '70c1db1f-b701-45bd-96e0-a313ee3430b3',
                'tenant_id': '',
                'extra_dhcp_opts': [],
                'binding:vif_details': {
                    'port_filter': True,
                    'ovs_hybrid_plug': True,
                },
                'binding:vif_type': 'ovs',
                'device_owner': 'network:router_gateway',
                'mac_address': 'fa:16:3e:58:42:ed',
                'binding:profile': {},
                'binding:vnic_type': 'normal',
                'fixed_ips': [
                    {
                        'subnet_id': '008ba151-0b8c-4a67-98b5-0d2b87666062',
                        'ip_address': '172.24.4.2',
                    }
                ],
                'id': 'd80b1a3b-4fc1-49f3-952e-1e2ab7081d8b',
                'security_groups': [],
                'device_id': '9ae135f4-b6e0-4dad-9e91-3c223e385824',
            },
            {
                'status': 'ACTIVE',
                'binding:host_id': 'devstack',
                'name': '',
                'allowed_address_pairs': [],
                'admin_state_up': True,
                'network_id': 'f27aa545-cbdd-4907-b0c6-c9e8b039dcc2',
                'tenant_id': 'd397de8a63f341818f198abb0966f6f3',
                'extra_dhcp_opts': [],
                'binding:vif_details': {
                    'port_filter': True,
                    'ovs_hybrid_plug': True,
                },
                'binding:vif_type': 'ovs',
                'device_owner': 'network:router_interface',
                'mac_address': 'fa:16:3e:bb:3c:e4',
                'binding:profile': {},
                'binding:vnic_type': 'normal',
                'fixed_ips': [
                    {
                        'subnet_id': '288bf4a1-51ba-43b6-9d0a-520e9005db17',
                        'ip_address': '10.0.0.1',
                    }
                ],
                'id': 'f71a6703-d6de-4be1-a91a-a570ede1d159',
                'security_groups': [],
                'device_id': '9ae135f4-b6e0-4dad-9e91-3c223e385824',
            },
        ]
    }

    def _compare_ports(self, exp, real):
        self.assertDictEqual(
            _port.Port(**exp).to_dict(computed=False),
            real.to_dict(computed=False),
        )

    def test_create_port(self):
        self.register_uris(
            [
                dict(
                    method="POST",
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'ports']
                    ),
                    json=self.mock_neutron_port_create_rep,
                    validate=dict(
                        json={
                            'port': {
                                'network_id': 'test-net-id',
                                'name': 'test-port-name',
                                'admin_state_up': True,
                            }
                        }
                    ),
                )
            ]
        )
        port = self.cloud.create_port(
            network_id='test-net-id',
            name='test-port-name',
            admin_state_up=True,
        )
        self._compare_ports(self.mock_neutron_port_create_rep['port'], port)
        self.assert_calls()

    def test_create_port_parameters(self):
        """Test that we detect invalid arguments passed to create_port"""
        self.assertRaises(
            TypeError,
            self.cloud.create_port,
            network_id='test-net-id',
            nome='test-port-name',
            stato_amministrativo_porta=True,
        )

    def test_create_port_exception(self):
        self.register_uris(
            [
                dict(
                    method="POST",
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'ports']
                    ),
                    status_code=500,
                    validate=dict(
                        json={
                            'port': {
                                'network_id': 'test-net-id',
                                'name': 'test-port-name',
                                'admin_state_up': True,
                            }
                        }
                    ),
                )
            ]
        )
        self.assertRaises(
            exceptions.SDKException,
            self.cloud.create_port,
            network_id='test-net-id',
            name='test-port-name',
            admin_state_up=True,
        )
        self.assert_calls()

    def test_create_port_with_project(self):
        self.mock_neutron_port_create_rep["port"].update(
            {
                'project_id': 'test-project-id',
            }
        )
        self.register_uris(
            [
                dict(
                    method="POST",
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'ports']
                    ),
                    json=self.mock_neutron_port_create_rep,
                    validate=dict(
                        json={
                            'port': {
                                'network_id': 'test-net-id',
                                'project_id': 'test-project-id',
                                'name': 'test-port-name',
                                'admin_state_up': True,
                            }
                        }
                    ),
                )
            ]
        )
        port = self.cloud.create_port(
            network_id='test-net-id',
            name='test-port-name',
            admin_state_up=True,
            project_id='test-project-id',
        )
        self._compare_ports(self.mock_neutron_port_create_rep['port'], port)
        self.assert_calls()

    def test_update_port(self):
        port_id = 'd80b1a3b-4fc1-49f3-952e-1e2ab7081d8b'
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'ports', port_id]
                    ),
                    json=dict(
                        port=self.mock_neutron_port_list_rep['ports'][0]
                    ),
                ),
                dict(
                    method='PUT',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'ports', port_id]
                    ),
                    json=self.mock_neutron_port_update_rep,
                    validate=dict(
                        json={'port': {'name': 'test-port-name-updated'}}
                    ),
                ),
            ]
        )
        port = self.cloud.update_port(
            name_or_id=port_id, name='test-port-name-updated'
        )

        self._compare_ports(self.mock_neutron_port_update_rep['port'], port)
        self.assert_calls()

    def test_update_port_parameters(self):
        """Test that we detect invalid arguments passed to update_port"""
        self.assertRaises(
            TypeError,
            self.cloud.update_port,
            name_or_id='test-port-id',
            nome='test-port-name-updated',
        )

    def test_update_port_exception(self):
        port_id = 'd80b1a3b-4fc1-49f3-952e-1e2ab7081d8b'
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'ports', port_id]
                    ),
                    json=self.mock_neutron_port_list_rep,
                ),
                dict(
                    method='PUT',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'ports', port_id]
                    ),
                    status_code=500,
                    validate=dict(
                        json={'port': {'name': 'test-port-name-updated'}}
                    ),
                ),
            ]
        )
        self.assertRaises(
            exceptions.SDKException,
            self.cloud.update_port,
            name_or_id='d80b1a3b-4fc1-49f3-952e-1e2ab7081d8b',
            name='test-port-name-updated',
        )
        self.assert_calls()

    def test_list_ports(self):
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'ports']
                    ),
                    json=self.mock_neutron_port_list_rep,
                )
            ]
        )
        ports = self.cloud.list_ports()
        for a, b in zip(self.mock_neutron_port_list_rep['ports'], ports):
            self._compare_ports(a, b)
        self.assert_calls()

    def test_list_ports_filtered(self):
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'ports'],
                        qs_elements=['status=DOWN'],
                    ),
                    json=self.mock_neutron_port_list_rep,
                )
            ]
        )
        ports = self.cloud.list_ports(filters={'status': 'DOWN'})
        for a, b in zip(self.mock_neutron_port_list_rep['ports'], ports):
            self._compare_ports(a, b)
        self.assert_calls()

    def test_list_ports_exception(self):
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'ports']
                    ),
                    status_code=500,
                )
            ]
        )
        self.assertRaises(exceptions.SDKException, self.cloud.list_ports)

    def test_search_ports_by_id(self):
        port_id = 'f71a6703-d6de-4be1-a91a-a570ede1d159'
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'ports']
                    ),
                    json=self.mock_neutron_port_list_rep,
                )
            ]
        )
        ports = self.cloud.search_ports(name_or_id=port_id)

        self.assertEqual(1, len(ports))
        self.assertEqual('fa:16:3e:bb:3c:e4', ports[0]['mac_address'])
        self.assert_calls()

    def test_search_ports_by_name(self):
        port_name = "first-port"
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'ports']
                    ),
                    json=self.mock_neutron_port_list_rep,
                )
            ]
        )
        ports = self.cloud.search_ports(name_or_id=port_name)

        self.assertEqual(1, len(ports))
        self.assertEqual('fa:16:3e:58:42:ed', ports[0]['mac_address'])
        self.assert_calls()

    def test_search_ports_not_found(self):
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'ports']
                    ),
                    json=self.mock_neutron_port_list_rep,
                )
            ]
        )
        ports = self.cloud.search_ports(name_or_id='non-existent')
        self.assertEqual(0, len(ports))
        self.assert_calls()

    def test_delete_port(self):
        port_id = 'd80b1a3b-4fc1-49f3-952e-1e2ab7081d8b'
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'ports', 'first-port'],
                    ),
                    status_code=404,
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'ports'],
                        qs_elements=['name=first-port'],
                    ),
                    json=self.mock_neutron_port_list_rep,
                ),
                dict(
                    method='DELETE',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'ports', port_id]
                    ),
                    json={},
                ),
            ]
        )

        self.assertTrue(self.cloud.delete_port(name_or_id='first-port'))

    def test_delete_port_not_found(self):
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'ports', 'non-existent'],
                    ),
                    status_code=404,
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'ports'],
                        qs_elements=['name=non-existent'],
                    ),
                    json={'ports': []},
                ),
            ]
        )
        self.assertFalse(self.cloud.delete_port(name_or_id='non-existent'))
        self.assert_calls()

    def test_delete_subnet_multiple_found(self):
        port_name = "port-name"
        port1 = dict(id='123', name=port_name)
        port2 = dict(id='456', name=port_name)
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'ports', port_name],
                    ),
                    status_code=404,
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'ports'],
                        qs_elements=['name=%s' % port_name],
                    ),
                    json={'ports': [port1, port2]},
                ),
            ]
        )
        self.assertRaises(
            exceptions.SDKException, self.cloud.delete_port, port_name
        )
        self.assert_calls()

    def test_delete_subnet_multiple_using_id(self):
        port_name = "port-name"
        port1 = dict(id='123', name=port_name)
        port2 = dict(id='456', name=port_name)
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'ports', port1['id']],
                    ),
                    json={'ports': [port1, port2]},
                ),
                dict(
                    method='DELETE',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'ports', port1['id']],
                    ),
                    json={},
                ),
            ]
        )
        self.assertTrue(self.cloud.delete_port(name_or_id=port1['id']))
        self.assert_calls()

    def test_get_port_by_id(self):
        fake_port = dict(id='123', name='456')
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'ports', fake_port['id']],
                    ),
                    json={'port': fake_port},
                )
            ]
        )
        r = self.cloud.get_port_by_id(fake_port['id'])
        self.assertIsNotNone(r)
        self._compare_ports(fake_port, r)
        self.assert_calls()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock
import uuid

import testtools

from openstack import connection
from openstack import exceptions
from openstack.tests.unit import base
from openstack import utils


RANGE_DATA = [
    dict(id=1, key1=1, key2=5),
    dict(id=2, key1=1, key2=20),
    dict(id=3, key1=2, key2=10),
    dict(id=4, key1=2, key2=30),
    dict(id=5, key1=3, key2=40),
    dict(id=6, key1=3, key2=40),
]


class TestCloud(base.TestCase):
    def test_openstack_cloud(self):
        self.assertIsInstance(self.cloud, connection.Connection)

    def test_endpoint_for(self):
        dns_override = 'https://override.dns.example.com'
        self.cloud.config.config['dns_endpoint_override'] = dns_override
        self.assertEqual(
            'https://compute.example.com/v2.1/',
            self.cloud.endpoint_for('compute'),
        )
        self.assertEqual(
            'https://internal.compute.example.com/v2.1/',
            self.cloud.endpoint_for('compute', interface='internal'),
        )
        self.assertIsNone(
            self.cloud.endpoint_for('compute', region_name='unknown-region')
        )
        self.assertEqual(dns_override, self.cloud.endpoint_for('dns'))

    def test_connect_as(self):
        # Do initial auth/catalog steps
        # This should authenticate a second time, but should not
        # need a second identity discovery
        project_name = 'test_project'
        self.register_uris(
            [
                self.get_keystone_v3_token(project_name=project_name),
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', 'detail']
                    ),
                    json={'servers': []},
                ),
            ]
        )

        c2 = self.cloud.connect_as(project_name=project_name)
        self.assertEqual(c2.list_servers(), [])
        self.assert_calls()

    def test_connect_as_context(self):
        # Do initial auth/catalog steps
        # This should authenticate a second time, but should not
        # need a second identity discovery
        project_name = 'test_project'
        self.register_uris(
            [
                self.get_keystone_v3_token(project_name=project_name),
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', 'detail']
                    ),
                    json={'servers': []},
                ),
            ]
        )

        with self.cloud.connect_as(project_name=project_name) as c2:
            self.assertEqual(c2.list_servers(), [])
        self.assert_calls()

    def test_global_request_id(self):
        request_id = uuid.uuid4().hex
        self.register_uris(
            [
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', 'detail']
                    ),
                    json={'servers': []},
                    validate=dict(
                        headers={'X-Openstack-Request-Id': request_id}
                    ),
                ),
            ]
        )

        cloud2 = self.cloud.global_request(request_id)
        self.assertEqual([], cloud2.list_servers())

        self.assert_calls()

    def test_global_request_id_context(self):
        request_id = uuid.uuid4().hex
        self.register_uris(
            [
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', 'detail']
                    ),
                    json={'servers': []},
                    validate=dict(
                        headers={'X-Openstack-Request-Id': request_id}
                    ),
                ),
            ]
        )

        with self.cloud.global_request(request_id) as c2:
            self.assertEqual([], c2.list_servers())

        self.assert_calls()

    def test_iterate_timeout_bad_wait(self):
        with testtools.ExpectedException(
            exceptions.SDKException,
            "Wait value must be an int or float value.",
        ):
            for count in utils.iterate_timeout(
                1, "test_iterate_timeout_bad_wait", wait="timeishard"
            ):
                pass

    @mock.patch('time.sleep')
    def test_iterate_timeout_str_wait(self, mock_sleep):
        iter = utils.iterate_timeout(
            10, "test_iterate_timeout_str_wait", wait="1.6"
        )
        next(iter)
        next(iter)
        mock_sleep.assert_called_with(1.6)

    @mock.patch('time.sleep')
    def test_iterate_timeout_int_wait(self, mock_sleep):
        iter = utils.iterate_timeout(
            10, "test_iterate_timeout_int_wait", wait=1
        )
        next(iter)
        next(iter)
        mock_sleep.assert_called_with(1.0)

    @mock.patch('time.sleep')
    def test_iterate_timeout_timeout(self, mock_sleep):
        message = "timeout test"
        with testtools.ExpectedException(exceptions.ResourceTimeout, message):
            for count in utils.iterate_timeout(0.1, message, wait=1):
                pass
        mock_sleep.assert_called_with(1.0)

    def test_range_search(self):
        filters = {"key1": "min", "key2": "20"}
        retval = self.cloud.range_search(RANGE_DATA, filters)
        self.assertIsInstance(retval, list)
        self.assertEqual(1, len(retval))
        self.assertEqual([RANGE_DATA[1]], retval)

    def test_range_search_2(self):
        filters = {"key1": "<=2", "key2": ">10"}
        retval = self.cloud.range_search(RANGE_DATA, filters)
        self.assertIsInstance(retval, list)
        self.assertEqual(2, len(retval))
        self.assertEqual([RANGE_DATA[1], RANGE_DATA[3]], retval)

    def test_range_search_3(self):
        filters = {"key1": "2", "key2": "min"}
        retval = self.cloud.range_search(RANGE_DATA, filters)
        self.assertIsInstance(retval, list)
        self.assertEqual(0, len(retval))

    def test_range_search_4(self):
        filters = {"key1": "max", "key2": "min"}
        retval = self.cloud.range_search(RANGE_DATA, filters)
        self.assertIsInstance(retval, list)
        self.assertEqual(0, len(retval))

    def test_range_search_5(self):
        filters = {"key1": "min", "key2": "min"}
        retval = self.cloud.range_search(RANGE_DATA, filters)
        self.assertIsInstance(retval, list)
        self.assertEqual(1, len(retval))
        self.assertEqual([RANGE_DATA[0]], retval)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.network.v2 import bgpvpn
from openstack.network.v2 import bgpvpn_network_association
from openstack.network.v2 import bgpvpn_port_association
from openstack.network.v2 import bgpvpn_router_association
from openstack.network.v2 import network
from openstack.network.v2 import port
from openstack.network.v2 import router
from openstack.tests.unit import base

IDENTIFIER = 'IDENTIFIER'
NET_ID = 'NET_ID'
PORT_ID = 'PORT_ID'
ROUTER_ID = 'ROUTER_ID'
EXAMPLE = {
    'id': IDENTIFIER,
    'name': 'bgpvpn',
    'project_id': '42',
    'route_distinguishers': ['64512:1777', '64512:1888', '64512:1999'],
    'route_targets': '64512:1444',
    'import_targets': '64512:1555',
    'export_targets': '64512:1666',
}


class TestBgpVpn(base.TestCase):
    def test_basic(self):
        sot = bgpvpn.BgpVpn()
        self.assertEqual('bgpvpn', sot.resource_key)
        self.assertEqual('bgpvpns', sot.resources_key)
        self.assertEqual('/bgpvpn/bgpvpns', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = bgpvpn.BgpVpn(**EXAMPLE)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['project_id'], sot.project_id)
        self.assertEqual(
            EXAMPLE['route_distinguishers'], sot.route_distinguishers
        )
        self.assertEqual(EXAMPLE['route_targets'], sot.route_targets)
        self.assertEqual(EXAMPLE['import_targets'], sot.import_targets)
        self.assertEqual(EXAMPLE['export_targets'], sot.export_targets)

        self.assertDictEqual(
            {
                'limit': 'limit',
                'marker': 'marker',
            },
            sot._query_mapping._mapping,
        )

    def test_create_bgpvpn_network_association(self):
        test_bpgvpn = bgpvpn.BgpVpn(**EXAMPLE)
        test_net = network.Network(**{'name': 'foo_net', 'id': NET_ID})
        sot = bgpvpn_network_association.BgpVpnNetworkAssociation(
            bgpvn_id=test_bpgvpn.id, network_id=test_net.id
        )
        self.assertEqual(test_net.id, sot.network_id)
        self.assertEqual(test_bpgvpn.id, sot.bgpvn_id)

    def test_create_bgpvpn_port_association(self):
        test_bpgvpn = bgpvpn.BgpVpn(**EXAMPLE)
        test_port = port.Port(
            **{'name': 'foo_port', 'id': PORT_ID, 'network_id': NET_ID}
        )
        sot = bgpvpn_port_association.BgpVpnPortAssociation(
            bgpvn_id=test_bpgvpn.id, port_id=test_port.id
        )
        self.assertEqual(test_port.id, sot.port_id)
        self.assertEqual(test_bpgvpn.id, sot.bgpvn_id)

    def test_create_bgpvpn_router_association(self):
        test_bpgvpn = bgpvpn.BgpVpn(**EXAMPLE)
        test_router = router.Router(**{'name': 'foo_port'})
        sot = bgpvpn_router_association.BgpVpnRouterAssociation(
            bgpvn_id=test_bpgvpn.id, router_id=test_router.id
        )
        self.assertEqual(test_router.id, sot.router_id)
        self.assertEqual(test_bpgvpn.id, sot.bgpvn_id)



# Copyright (c) 2018, Intel Corporation.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.network.v2 import network_segment_range
from openstack.tests.unit import base


IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    'id': IDENTIFIER,
    'name': '1',
    'default': False,
    'shared': False,
    'project_id': '2',
    'network_type': '3',
    'physical_network': '4',
    'minimum': 5,
    'maximum': 6,
    'used': {},
    'available': [],
}


class TestNetworkSegmentRange(base.TestCase):
    def test_basic(self):
        test_seg_range = network_segment_range.NetworkSegmentRange()
        self.assertEqual('network_segment_range', test_seg_range.resource_key)
        self.assertEqual(
            'network_segment_ranges', test_seg_range.resources_key
        )
        self.assertEqual('/network_segment_ranges', test_seg_range.base_path)

        self.assertTrue(test_seg_range.allow_create)
        self.assertTrue(test_seg_range.allow_fetch)
        self.assertTrue(test_seg_range.allow_commit)
        self.assertTrue(test_seg_range.allow_delete)
        self.assertTrue(test_seg_range.allow_list)

    def test_make_it(self):
        test_seg_range = network_segment_range.NetworkSegmentRange(**EXAMPLE)
        self.assertEqual(EXAMPLE['id'], test_seg_range.id)
        self.assertEqual(EXAMPLE['name'], test_seg_range.name)
        self.assertEqual(EXAMPLE['default'], test_seg_range.default)
        self.assertEqual(EXAMPLE['shared'], test_seg_range.shared)
        self.assertEqual(EXAMPLE['project_id'], test_seg_range.project_id)
        self.assertEqual(EXAMPLE['network_type'], test_seg_range.network_type)
        self.assertEqual(
            EXAMPLE['physical_network'], test_seg_range.physical_network
        )
        self.assertEqual(EXAMPLE['minimum'], test_seg_range.minimum)
        self.assertEqual(EXAMPLE['maximum'], test_seg_range.maximum)
        self.assertEqual(EXAMPLE['used'], test_seg_range.used)
        self.assertEqual(EXAMPLE['available'], test_seg_range.available)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
test_delete_volume_snapshot
----------------------------------

Tests for the `delete_volume_snapshot` command.
"""

from openstack.cloud import meta
from openstack import exceptions
from openstack.tests import fakes
from openstack.tests.unit import base


class TestDeleteVolumeSnapshot(base.TestCase):
    def setUp(self):
        super(TestDeleteVolumeSnapshot, self).setUp()
        self.use_cinder()

    def test_delete_volume_snapshot(self):
        """
        Test that delete_volume_snapshot without a wait returns True instance
        when the volume snapshot deletes.
        """
        fake_snapshot = fakes.FakeVolumeSnapshot(
            '1234', 'available', 'foo', 'derpysnapshot'
        )
        fake_snapshot_dict = meta.obj_to_munch(fake_snapshot)

        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'volumev3', 'public', append=['snapshots', 'detail']
                    ),
                    json={'snapshots': [fake_snapshot_dict]},
                ),
                dict(
                    method='DELETE',
                    uri=self.get_mock_url(
                        'volumev3',
                        'public',
                        append=['snapshots', fake_snapshot_dict['id']],
                    ),
                ),
            ]
        )

        self.assertTrue(
            self.cloud.delete_volume_snapshot(name_or_id='1234', wait=False)
        )
        self.assert_calls()

    def test_delete_volume_snapshot_with_error(self):
        """
        Test that a exception while deleting a volume snapshot will cause an
        SDKException.
        """
        fake_snapshot = fakes.FakeVolumeSnapshot(
            '1234', 'available', 'foo', 'derpysnapshot'
        )
        fake_snapshot_dict = meta.obj_to_munch(fake_snapshot)
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'volumev3', 'public', append=['snapshots', 'detail']
                    ),
                    json={'snapshots': [fake_snapshot_dict]},
                ),
                dict(
                    method='DELETE',
                    uri=self.get_mock_url(
                        'volumev3',
                        'public',
                        append=['snapshots', fake_snapshot_dict['id']],
                    ),
                    status_code=404,
                ),
            ]
        )

        self.assertRaises(
            exceptions.SDKException,
            self.cloud.delete_volume_snapshot,
            name_or_id='1234',
        )
        self.assert_calls()

    def test_delete_volume_snapshot_with_timeout(self):
        """
        Test that a timeout while waiting for the volume snapshot to delete
        raises an exception in delete_volume_snapshot.
        """
        fake_snapshot = fakes.FakeVolumeSnapshot(
            '1234', 'available', 'foo', 'derpysnapshot'
        )
        fake_snapshot_dict = meta.obj_to_munch(fake_snapshot)

        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'volumev3', 'public', append=['snapshots', 'detail']
                    ),
                    json={'snapshots': [fake_snapshot_dict]},
                ),
                dict(
                    method='DELETE',
                    uri=self.get_mock_url(
                        'volumev3',
                        'public',
                        append=['snapshots', fake_snapshot_dict['id']],
                    ),
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'volumev3', 'public', append=['snapshots', '1234']
                    ),
                    json={'snapshot': fake_snapshot_dict},
                ),
            ]
        )

        self.assertRaises(
            exceptions.ResourceTimeout,
            self.cloud.delete_volume_snapshot,
            name_or_id='1234',
            wait=True,
            timeout=0.01,
        )
        self.assert_calls(do_count=False)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import exceptions
from openstack import resource
from openstack import utils


class Service(resource.Resource):
    resource_key = 'service'
    resources_key = 'services'
    base_path = '/os-services'

    # capabilities
    allow_list = True
    allow_commit = True
    allow_delete = True

    _query_mapping = resource.QueryParameters(
        'name',
        'binary',
        'host',
        name='binary',
    )

    # Properties
    #: The availability zone of service
    availability_zone = resource.Body("zone")
    #: Binary name of service
    binary = resource.Body('binary')
    #: Disabled reason of service
    disabled_reason = resource.Body('disabled_reason')
    #: Whether or not this service was forced down manually by an administrator
    #: after the service was fenced
    is_forced_down = resource.Body('forced_down', type=bool)
    #: The name of the host where service runs
    host = resource.Body('host')
    #: Service name
    name = resource.Body('name', alias='binary')
    #: State of service
    state = resource.Body('state')
    #: Status of service
    status = resource.Body('status')
    #: The date and time when the resource was updated
    updated_at = resource.Body('updated_at')

    _max_microversion = '2.69'

    @classmethod
    def find(cls, session, name_or_id, ignore_missing=True, **params):
        # No direct request possible, thus go directly to list
        data = cls.list(session, **params)

        result = None
        for maybe_result in data:
            # Since ID might be both int and str force cast
            id_value = str(cls._get_id(maybe_result))
            name_value = maybe_result.name

            if str(name_or_id) in (id_value, name_value):
                if 'host' in params and maybe_result['host'] != params['host']:
                    continue
                # Only allow one resource to be found. If we already
                # found a match, raise an exception to show it.
                if result is None:
                    result = maybe_result
                else:
                    msg = "More than one %s exists with the name '%s'."
                    msg = msg % (cls.__name__, name_or_id)
                    raise exceptions.DuplicateResource(msg)

        if result is not None:
            return result

        if ignore_missing:
            return None
        raise exceptions.ResourceNotFound(
            "No %s found for %s" % (cls.__name__, name_or_id)
        )

    def commit(self, session, prepend_key=False, **kwargs):
        # we need to set prepend_key to false
        return super(Service, self).commit(
            session,
            prepend_key=prepend_key,
            **kwargs,
        )

    def _action(self, session, action, body, microversion=None):
        if not microversion:
            microversion = session.default_microversion
        url = utils.urljoin(Service.base_path, action)
        response = session.put(url, json=body, microversion=microversion)
        self._translate_response(response)
        return self

    def set_forced_down(self, session, host=None, binary=None, forced=False):
        """Update forced_down information of a service."""
        microversion = session.default_microversion
        body = {}
        if not host:
            host = self.host
        if not binary:
            binary = self.binary
        body = {
            'host': host,
            'binary': binary,
        }
        if utils.supports_microversion(session, '2.11'):
            body['forced_down'] = forced
            # Using forced_down works only 2.11-2.52, therefore pin it
            microversion = '2.11'

        # This will not work with newest microversions
        return self._action(
            session,
            'force-down',
            body,
            microversion=microversion,
        )

    force_down = set_forced_down

    def enable(self, session, host, binary):
        """Enable service."""
        body = {
            'host': host,
            'binary': binary,
        }

        return self._action(session, 'enable', body)

    def disable(self, session, host, binary, reason=None):
        """Disable service."""
        body = {
            'host': host,
            'binary': binary,
        }

        if not reason:
            action = 'disable'
        else:
            body['disabled_reason'] = reason
            action = 'disable-log-reason'

        return self._action(session, action, body)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.key_manager.v1 import container
from openstack.tests.unit import base


ID_VAL = "123"
IDENTIFIER = 'http://localhost/containers/%s' % ID_VAL
EXAMPLE = {
    'container_ref': IDENTIFIER,
    'created': '2015-03-09T12:14:57.233772',
    'name': '3',
    'secret_refs': ['4'],
    'status': '5',
    'type': '6',
    'updated': '2015-03-09T12:15:57.233772',
    'consumers': ['7'],
}


class TestContainer(base.TestCase):
    def test_basic(self):
        sot = container.Container()
        self.assertIsNone(sot.resource_key)
        self.assertEqual('containers', sot.resources_key)
        self.assertEqual('/containers', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = container.Container(**EXAMPLE)
        self.assertEqual(EXAMPLE['created'], sot.created_at)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['secret_refs'], sot.secret_refs)
        self.assertEqual(EXAMPLE['status'], sot.status)
        self.assertEqual(EXAMPLE['type'], sot.type)
        self.assertEqual(EXAMPLE['updated'], sot.updated_at)
        self.assertEqual(EXAMPLE['container_ref'], sot.id)
        self.assertEqual(EXAMPLE['container_ref'], sot.container_ref)
        self.assertEqual(ID_VAL, sot.container_id)
        self.assertEqual(EXAMPLE['consumers'], sot.consumers)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.image.v2 import metadef_schema as _metadef_schema
from openstack.tests.functional.image.v2 import base


class TestMetadefSchema(base.BaseImageTest):
    def test_get_metadef_namespace_schema(self):
        metadef_schema = self.conn.image.get_metadef_namespace_schema()
        self.assertIsNotNone(metadef_schema)
        self.assertIsInstance(metadef_schema, _metadef_schema.MetadefSchema)

    def test_get_metadef_namespaces_schema(self):
        metadef_schema = self.conn.image.get_metadef_namespaces_schema()
        self.assertIsNotNone(metadef_schema)
        self.assertIsInstance(metadef_schema, _metadef_schema.MetadefSchema)

    def test_get_metadef_resource_type_schema(self):
        metadef_schema = self.conn.image.get_metadef_resource_type_schema()
        self.assertIsNotNone(metadef_schema)
        self.assertIsInstance(metadef_schema, _metadef_schema.MetadefSchema)

    def test_get_metadef_resource_types_schema(self):
        metadef_schema = self.conn.image.get_metadef_resource_types_schema()
        self.assertIsNotNone(metadef_schema)
        self.assertIsInstance(metadef_schema, _metadef_schema.MetadefSchema)

    def test_get_metadef_object_schema(self):
        metadef_schema = self.conn.image.get_metadef_object_schema()
        self.assertIsNotNone(metadef_schema)
        self.assertIsInstance(metadef_schema, _metadef_schema.MetadefSchema)

    def test_get_metadef_objects_schema(self):
        metadef_schema = self.conn.image.get_metadef_objects_schema()
        self.assertIsNotNone(metadef_schema)
        self.assertIsInstance(metadef_schema, _metadef_schema.MetadefSchema)

    def test_get_metadef_property_schema(self):
        metadef_schema = self.conn.image.get_metadef_property_schema()
        self.assertIsNotNone(metadef_schema)
        self.assertIsInstance(metadef_schema, _metadef_schema.MetadefSchema)

    def test_get_metadef_properties_schema(self):
        metadef_schema = self.conn.image.get_metadef_properties_schema()
        self.assertIsNotNone(metadef_schema)
        self.assertIsInstance(metadef_schema, _metadef_schema.MetadefSchema)

    def test_get_metadef_tag_schema(self):
        metadef_schema = self.conn.image.get_metadef_tag_schema()
        self.assertIsNotNone(metadef_schema)
        self.assertIsInstance(metadef_schema, _metadef_schema.MetadefSchema)

    def test_get_metadef_tags_schema(self):
        metadef_schema = self.conn.image.get_metadef_tags_schema()
        self.assertIsNotNone(metadef_schema)
        self.assertIsInstance(metadef_schema, _metadef_schema.MetadefSchema)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import testtools

from openstack.block_storage.v3 import volume
from openstack.cloud import meta
from openstack.compute.v2 import volume_attachment
from openstack import exceptions
from openstack.tests import fakes
from openstack.tests.unit import base


class TestVolume(base.TestCase):
    def _compare_volumes(self, exp, real):
        self.assertDictEqual(
            volume.Volume(**exp).to_dict(computed=False),
            real.to_dict(computed=False),
        )

    def _compare_volume_attachments(self, exp, real):
        self.assertDictEqual(
            volume_attachment.VolumeAttachment(**exp).to_dict(computed=False),
            real.to_dict(computed=False),
        )

    def test_attach_volume(self):
        server = dict(id='server001')
        vol = {
            'id': 'volume001',
            'status': 'available',
            'name': '',
            'attachments': [],
        }
        volume = meta.obj_to_munch(fakes.FakeVolume(**vol))
        rattach = {
            'server_id': server['id'],
            'device': 'device001',
            'volumeId': volume['id'],
            'id': 'attachmentId',
        }
        self.register_uris(
            [
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'compute',
                        'public',
                        append=[
                            'servers',
                            server['id'],
                            'os-volume_attachments',
                        ],
                    ),
                    json={'volumeAttachment': rattach},
                    validate=dict(
                        json={'volumeAttachment': {'volumeId': vol['id']}}
                    ),
                ),
            ]
        )
        ret = self.cloud.attach_volume(server, volume, wait=False)
        self._compare_volume_attachments(rattach, ret)
        self.assert_calls()

    def test_attach_volume_exception(self):
        server = dict(id='server001')
        vol = {
            'id': 'volume001',
            'status': 'available',
            'name': '',
            'attachments': [],
        }
        volume = meta.obj_to_munch(fakes.FakeVolume(**vol))
        self.register_uris(
            [
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'compute',
                        'public',
                        append=[
                            'servers',
                            server['id'],
                            'os-volume_attachments',
                        ],
                    ),
                    status_code=404,
                    validate=dict(
                        json={'volumeAttachment': {'volumeId': vol['id']}}
                    ),
                ),
            ]
        )
        with testtools.ExpectedException(
            exceptions.NotFoundException,
        ):
            self.cloud.attach_volume(server, volume, wait=False)
        self.assert_calls()

    def test_attach_volume_wait(self):
        server = dict(id='server001')
        vol = {
            'id': 'volume001',
            'status': 'available',
            'name': '',
            'attachments': [],
        }
        volume = meta.obj_to_munch(fakes.FakeVolume(**vol))
        vol['attachments'] = [
            {'server_id': server['id'], 'device': 'device001'}
        ]
        vol['status'] = 'in-use'
        attached_volume = meta.obj_to_munch(fakes.FakeVolume(**vol))
        rattach = {
            'server_id': server['id'],
            'device': 'device001',
            'volumeId': volume['id'],
            'id': 'attachmentId',
        }
        self.register_uris(
            [
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'compute',
                        'public',
                        append=[
                            'servers',
                            server['id'],
                            'os-volume_attachments',
                        ],
                    ),
                    json={'volumeAttachment': rattach},
                    validate=dict(
                        json={'volumeAttachment': {'volumeId': vol['id']}}
                    ),
                ),
                self.get_cinder_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'volumev3', 'public', append=['volumes', vol['id']]
                    ),
                    json={'volume': volume},
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'volumev3', 'public', append=['volumes', vol['id']]
                    ),
                    json={'volume': attached_volume},
                ),
            ]
        )
        # defaults to wait=True
        ret = self.cloud.attach_volume(server, volume)
        self._compare_volume_attachments(rattach, ret)
        self.assert_calls()

    def test_attach_volume_wait_error(self):
        server = dict(id='server001')
        vol = {
            'id': 'volume001',
            'status': 'available',
            'name': '',
            'attachments': [],
        }
        volume = meta.obj_to_munch(fakes.FakeVolume(**vol))
        vol['status'] = 'error'
        errored_volume = meta.obj_to_munch(fakes.FakeVolume(**vol))
        rattach = {
            'server_id': server['id'],
            'device': 'device001',
            'volumeId': volume['id'],
            'id': 'attachmentId',
        }
        self.register_uris(
            [
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'compute',
                        'public',
                        append=[
                            'servers',
                            server['id'],
                            'os-volume_attachments',
                        ],
                    ),
                    json={'volumeAttachment': rattach},
                    validate=dict(
                        json={'volumeAttachment': {'volumeId': vol['id']}}
                    ),
                ),
                self.get_cinder_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'volumev3', 'public', append=['volumes', volume['id']]
                    ),
                    json={'volume': errored_volume},
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'volumev3', 'public', append=['volumes', volume['id']]
                    ),
                    json={'volume': errored_volume},
                ),
            ]
        )

        with testtools.ExpectedException(exceptions.ResourceFailure):
            self.cloud.attach_volume(server, volume)
        self.assert_calls()

    def test_attach_volume_not_available(self):
        server = dict(id='server001')
        volume = dict(id='volume001', status='error', attachments=[])

        with testtools.ExpectedException(
            exceptions.SDKException,
            "Volume %s is not available. Status is '%s'"
            % (volume['id'], volume['status']),
        ):
            self.cloud.attach_volume(server, volume)
        self.assertEqual(0, len(self.adapter.request_history))

    def test_attach_volume_already_attached(self):
        device_id = 'device001'
        server = dict(id='server001')
        volume = dict(
            id='volume001',
            attachments=[{'server_id': 'server001', 'device': device_id}],
        )

        with testtools.ExpectedException(
            exceptions.SDKException,
            "Volume %s already attached to server %s on device %s"
            % (volume['id'], server['id'], device_id),
        ):
            self.cloud.attach_volume(server, volume)
        self.assertEqual(0, len(self.adapter.request_history))

    def test_detach_volume(self):
        server = dict(id='server001')
        volume = dict(
            id='volume001',
            attachments=[{'server_id': 'server001', 'device': 'device001'}],
        )
        self.register_uris(
            [
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', server['id']]
                    ),
                    json={'server': server},
                ),
                dict(
                    method='DELETE',
                    uri=self.get_mock_url(
                        'compute',
                        'public',
                        append=[
                            'servers',
                            server['id'],
                            'os-volume_attachments',
                            volume['id'],
                        ],
                    ),
                ),
            ]
        )
        self.cloud.detach_volume(server, volume, wait=False)
        self.assert_calls()

    def test_detach_volume_exception(self):
        server = dict(id='server001')
        volume = dict(
            id='volume001',
            attachments=[{'server_id': 'server001', 'device': 'device001'}],
        )
        self.register_uris(
            [
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', server['id']]
                    ),
                    json={'server': server},
                ),
                dict(
                    method='DELETE',
                    uri=self.get_mock_url(
                        'compute',
                        'public',
                        append=[
                            'servers',
                            server['id'],
                            'os-volume_attachments',
                            volume['id'],
                        ],
                    ),
                    status_code=404,
                ),
            ]
        )
        with testtools.ExpectedException(
            exceptions.NotFoundException,
        ):
            self.cloud.detach_volume(server, volume, wait=False)
        self.assert_calls()

    def test_detach_volume_wait(self):
        server = dict(id='server001')
        attachments = [{'server_id': 'server001', 'device': 'device001'}]
        vol = {
            'id': 'volume001',
            'status': 'attached',
            'name': '',
            'attachments': attachments,
        }
        volume = meta.obj_to_munch(fakes.FakeVolume(**vol))
        vol['status'] = 'available'
        vol['attachments'] = []
        avail_volume = meta.obj_to_munch(fakes.FakeVolume(**vol))
        self.register_uris(
            [
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', server['id']]
                    ),
                    json={'server': server},
                ),
                dict(
                    method='DELETE',
                    uri=self.get_mock_url(
                        'compute',
                        'public',
                        append=[
                            'servers',
                            server['id'],
                            'os-volume_attachments',
                            volume.id,
                        ],
                    ),
                ),
                self.get_cinder_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'volumev3', 'public', append=['volumes', 'detail']
                    ),
                    json={'volumes': [avail_volume]},
                ),
            ]
        )
        self.cloud.detach_volume(server, volume)
        self.assert_calls()

    def test_detach_volume_wait_error(self):
        server = dict(id='server001')
        attachments = [{'server_id': 'server001', 'device': 'device001'}]
        vol = {
            'id': 'volume001',
            'status': 'attached',
            'name': '',
            'attachments': attachments,
        }
        volume = meta.obj_to_munch(fakes.FakeVolume(**vol))
        vol['status'] = 'error'
        vol['attachments'] = []
        errored_volume = meta.obj_to_munch(fakes.FakeVolume(**vol))
        self.register_uris(
            [
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', server['id']]
                    ),
                    json={'server': server},
                ),
                dict(
                    method='DELETE',
                    uri=self.get_mock_url(
                        'compute',
                        'public',
                        append=[
                            'servers',
                            server['id'],
                            'os-volume_attachments',
                            volume.id,
                        ],
                    ),
                ),
                self.get_cinder_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'volumev3', 'public', append=['volumes', 'detail']
                    ),
                    json={'volumes': [errored_volume]},
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'volumev3',
                        'public',
                        append=['volumes', errored_volume['id']],
                    ),
                    json={'volume': errored_volume},
                ),
            ]
        )
        with testtools.ExpectedException(exceptions.ResourceFailure):
            self.cloud.detach_volume(server, volume)
        self.assert_calls()

    def test_delete_volume_deletes(self):
        vol = {
            'id': 'volume001',
            'status': 'attached',
            'name': '',
            'attachments': [],
        }
        volume = meta.obj_to_munch(fakes.FakeVolume(**vol))
        self.register_uris(
            [
                self.get_cinder_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'volumev3', 'public', append=['volumes', volume.id]
                    ),
                    json={'volumes': [volume]},
                ),
                dict(
                    method='DELETE',
                    uri=self.get_mock_url(
                        'volumev3', 'public', append=['volumes', volume.id]
                    ),
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'volumev3', 'public', append=['volumes', volume.id]
                    ),
                    status_code=404,
                ),
            ]
        )
        self.assertTrue(self.cloud.delete_volume(volume['id']))
        self.assert_calls()

    def test_delete_volume_gone_away(self):
        vol = {
            'id': 'volume001',
            'status': 'attached',
            'name': '',
            'attachments': [],
        }
        volume = meta.obj_to_munch(fakes.FakeVolume(**vol))
        self.register_uris(
            [
                self.get_cinder_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'volumev3', 'public', append=['volumes', volume.id]
                    ),
                    json=volume,
                ),
                dict(
                    method='DELETE',
                    uri=self.get_mock_url(
                        'volumev3', 'public', append=['volumes', volume.id]
                    ),
                    status_code=404,
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'volumev3', 'public', append=['volumes', volume.id]
                    ),
                    status_code=404,
                ),
            ]
        )
        self.assertTrue(self.cloud.delete_volume(volume['id']))
        self.assert_calls()

    def test_delete_volume_force(self):
        vol = {
            'id': 'volume001',
            'status': 'attached',
            'name': '',
            'attachments': [],
        }
        volume = meta.obj_to_munch(fakes.FakeVolume(**vol))
        self.register_uris(
            [
                self.get_cinder_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'volumev3', 'public', append=['volumes', volume['id']]
                    ),
                    json={'volumes': [volume]},
                ),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'volumev3',
                        'public',
                        append=['volumes', volume.id, 'action'],
                    ),
                    validate=dict(json={'os-force_delete': None}),
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'volumev3', 'public', append=['volumes', volume['id']]
                    ),
                    status_code=404,
                ),
            ]
        )
        self.assertTrue(self.cloud.delete_volume(volume['id'], force=True))
        self.assert_calls()

    def test_set_volume_bootable(self):
        vol = {
            'id': 'volume001',
            'status': 'attached',
            'name': '',
            'attachments': [],
        }
        volume = meta.obj_to_munch(fakes.FakeVolume(**vol))
        self.register_uris(
            [
                self.get_cinder_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'volumev3', 'public', append=['volumes', 'detail']
                    ),
                    json={'volumes': [volume]},
                ),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'volumev3',
                        'public',
                        append=['volumes', volume.id, 'action'],
                    ),
                    json={'os-set_bootable': {'bootable': True}},
                ),
            ]
        )
        self.cloud.set_volume_bootable(volume['id'])
        self.assert_calls()

    def test_set_volume_bootable_false(self):
        vol = {
            'id': 'volume001',
            'status': 'attached',
            'name': '',
            'attachments': [],
        }
        volume = meta.obj_to_munch(fakes.FakeVolume(**vol))
        self.register_uris(
            [
                self.get_cinder_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'volumev3', 'public', append=['volumes', 'detail']
                    ),
                    json={'volumes': [volume]},
                ),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'volumev3',
                        'public',
                        append=['volumes', volume.id, 'action'],
                    ),
                    json={'os-set_bootable': {'bootable': False}},
                ),
            ]
        )
        self.cloud.set_volume_bootable(volume['id'])
        self.assert_calls()

    def test_get_volume_by_id(self):
        vol1 = meta.obj_to_munch(fakes.FakeVolume('01', 'available', 'vol1'))
        self.register_uris(
            [
                self.get_cinder_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'volumev3', 'public', append=['volumes', '01']
                    ),
                    json={'volume': vol1},
                ),
            ]
        )
        self._compare_volumes(vol1, self.cloud.get_volume_by_id('01'))
        self.assert_calls()

    def test_create_volume(self):
        vol1 = meta.obj_to_munch(fakes.FakeVolume('01', 'available', 'vol1'))
        self.register_uris(
            [
                self.get_cinder_discovery_mock_dict(),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'volumev3', 'public', append=['volumes']
                    ),
                    json={'volume': vol1},
                    validate=dict(
                        json={
                            'volume': {
                                'size': 50,
                                'name': 'vol1',
                            }
                        }
                    ),
                ),
            ]
        )

        self.cloud.create_volume(50, name='vol1')
        self.assert_calls()

    def test_create_bootable_volume(self):
        vol1 = meta.obj_to_munch(fakes.FakeVolume('01', 'available', 'vol1'))
        self.register_uris(
            [
                self.get_cinder_discovery_mock_dict(),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'volumev3', 'public', append=['volumes']
                    ),
                    json={'volume': vol1},
                    validate=dict(
                        json={
                            'volume': {
                                'size': 50,
                                'name': 'vol1',
                            }
                        }
                    ),
                ),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'volumev3',
                        'public',
                        append=['volumes', '01', 'action'],
                    ),
                    validate=dict(
                        json={'os-set_bootable': {'bootable': True}}
                    ),
                ),
            ]
        )

        self.cloud.create_volume(50, name='vol1', bootable=True)
        self.assert_calls()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from openstack.clustering.v1 import node
from openstack.tests.unit import base

FAKE_ID = '123d0955-0099-aabb-b8fa-6a44655ceeff'
FAKE_NAME = 'test_node'

FAKE = {
    'id': FAKE_ID,
    'cluster_id': 'clusterA',
    'metadata': {'key1': 'value1'},
    'name': FAKE_NAME,
    'profile_id': 'myserver',
    'domain': '204ccccd267b40aea871750116b5b184',
    'user': '3747afc360b64702a53bdd64dc1b8976',
    'project': '42d9e9663331431f97b75e25136307ff',
    'index': 1,
    'role': 'master',
    'dependents': {},
    'created_at': '2015-10-10T12:46:36.000000',
    'updated_at': '2016-10-10T12:46:36.000000',
    'init_at': '2015-10-10T12:46:36.000000',
    'tainted': True,
}


class TestNode(base.TestCase):
    def test_basic(self):
        sot = node.Node()
        self.assertEqual('node', sot.resource_key)
        self.assertEqual('nodes', sot.resources_key)
        self.assertEqual('/nodes', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_instantiate(self):
        sot = node.Node(**FAKE)
        self.assertEqual(FAKE['id'], sot.id)
        self.assertEqual(FAKE['profile_id'], sot.profile_id)
        self.assertEqual(FAKE['cluster_id'], sot.cluster_id)
        self.assertEqual(FAKE['user'], sot.user_id)
        self.assertEqual(FAKE['project'], sot.project_id)
        self.assertEqual(FAKE['domain'], sot.domain_id)
        self.assertEqual(FAKE['name'], sot.name)
        self.assertEqual(FAKE['index'], sot.index)
        self.assertEqual(FAKE['role'], sot.role)
        self.assertEqual(FAKE['metadata'], sot.metadata)
        self.assertEqual(FAKE['init_at'], sot.init_at)
        self.assertEqual(FAKE['created_at'], sot.created_at)
        self.assertEqual(FAKE['updated_at'], sot.updated_at)
        self.assertEqual(FAKE['dependents'], sot.dependents)
        self.assertEqual(FAKE['tainted'], sot.tainted)

    def test_check(self):
        sot = node.Node(**FAKE)

        resp = mock.Mock()
        resp.json = mock.Mock(return_value='')
        sess = mock.Mock()
        sess.post = mock.Mock(return_value=resp)
        self.assertEqual('', sot.check(sess))
        url = 'nodes/%s/actions' % sot.id
        body = {'check': {}}
        sess.post.assert_called_once_with(url, json=body)

    def test_recover(self):
        sot = node.Node(**FAKE)

        resp = mock.Mock()
        resp.json = mock.Mock(return_value='')
        sess = mock.Mock()
        sess.post = mock.Mock(return_value=resp)
        self.assertEqual('', sot.recover(sess))
        url = 'nodes/%s/actions' % sot.id
        body = {'recover': {}}
        sess.post.assert_called_once_with(url, json=body)

    def test_operation(self):
        sot = node.Node(**FAKE)

        resp = mock.Mock()
        resp.json = mock.Mock(return_value='')
        sess = mock.Mock()
        sess.post = mock.Mock(return_value=resp)
        self.assertEqual('', sot.op(sess, 'dance', style='tango'))
        url = 'nodes/%s/ops' % sot.id
        sess.post.assert_called_once_with(
            url, json={'dance': {'style': 'tango'}}
        )

    def test_adopt_preview(self):
        sot = node.Node.new()
        resp = mock.Mock()
        resp.headers = {}
        resp.json = mock.Mock(return_value={"foo": "bar"})
        sess = mock.Mock()
        sess.post = mock.Mock(return_value=resp)

        attrs = {
            'identity': 'fake-resource-id',
            'overrides': {},
            'type': 'os.nova.server-1.0',
            'snapshot': False,
        }
        res = sot.adopt(sess, True, **attrs)
        self.assertEqual({"foo": "bar"}, res)
        sess.post.assert_called_once_with("nodes/adopt-preview", json=attrs)

    def test_adopt(self):
        sot = node.Node.new()
        resp = mock.Mock()
        resp.headers = {}
        resp.json = mock.Mock(return_value={"foo": "bar"})
        resp.status_code = 200
        sess = mock.Mock()
        sess.post = mock.Mock(return_value=resp)

        res = sot.adopt(sess, False, param="value")
        self.assertEqual(sot, res)
        sess.post.assert_called_once_with(
            "nodes/adopt", json={"param": "value"}
        )

    def test_force_delete(self):
        sot = node.Node(**FAKE)

        resp = mock.Mock()
        fake_action_id = 'f1de9847-2382-4272-8e73-cab0bc194663'
        resp.headers = {'Location': fake_action_id}
        resp.json = mock.Mock(return_value={"foo": "bar"})
        resp.status_code = 200
        sess = mock.Mock()
        sess.delete = mock.Mock(return_value=resp)

        res = sot.force_delete(sess)
        self.assertEqual(fake_action_id, res.id)
        url = 'nodes/%s' % sot.id
        body = {'force': True}
        sess.delete.assert_called_once_with(url, json=body)


class TestNodeDetail(base.TestCase):
    def test_basic(self):
        sot = node.NodeDetail()
        self.assertEqual('/nodes/%(node_id)s?show_details=True', sot.base_path)
        self.assertFalse(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertFalse(sot.allow_commit)
        self.assertFalse(sot.allow_delete)
        self.assertFalse(sot.allow_list)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from openstack.network.v2 import network
from openstack.tests.functional import base


def create_network(conn, name, cidr):
    try:
        network = conn.network.create_network(name=name)
        subnet = conn.network.create_subnet(
            name=name, ip_version=4, network_id=network.id, cidr=cidr
        )
        return (network, subnet)
    except Exception as e:
        print(str(e))
        pass
    return (None, None)


def delete_network(conn, network, subnet):
    if subnet:
        conn.network.delete_subnet(subnet)
    if network:
        conn.network.delete_network(network)


class TestNetwork(base.BaseFunctionalTest):
    ID = None

    def setUp(self):
        super(TestNetwork, self).setUp()
        self.NAME = self.getUniqueString()
        sot = self.user_cloud.network.create_network(name=self.NAME)
        assert isinstance(sot, network.Network)
        self.assertEqual(self.NAME, sot.name)
        self.ID = sot.id

    def tearDown(self):
        sot = self.user_cloud.network.delete_network(
            self.ID, ignore_missing=False
        )
        self.assertIsNone(sot)
        super(TestNetwork, self).tearDown()

    def test_find(self):
        sot = self.user_cloud.network.find_network(self.NAME)
        self.assertEqual(self.ID, sot.id)

    def test_find_with_filter(self):
        if not self.operator_cloud:
            self.skipTest("Operator cloud required for this test")
        project_id_1 = "1"
        project_id_2 = "2"
        sot1 = self.operator_cloud.network.create_network(
            name=self.NAME, project_id=project_id_1
        )
        sot2 = self.operator_cloud.network.create_network(
            name=self.NAME, project_id=project_id_2
        )
        sot = self.operator_cloud.network.find_network(
            self.NAME, project_id=project_id_1
        )
        self.assertEqual(project_id_1, sot.project_id)
        self.operator_cloud.network.delete_network(sot1.id)
        self.operator_cloud.network.delete_network(sot2.id)

    def test_get(self):
        sot = self.user_cloud.network.get_network(self.ID)
        self.assertEqual(self.NAME, sot.name)
        self.assertEqual(self.ID, sot.id)

    def test_list(self):
        names = [o.name for o in self.user_cloud.network.networks()]
        self.assertIn(self.NAME, names)

    def test_set_tags(self):
        sot = self.user_cloud.network.get_network(self.ID)
        self.assertEqual([], sot.tags)

        self.user_cloud.network.set_tags(sot, ["blue"])
        sot = self.user_cloud.network.get_network(self.ID)
        self.assertEqual(["blue"], sot.tags)

        self.user_cloud.network.set_tags(sot, [])
        sot = self.user_cloud.network.get_network(self.ID)
        self.assertEqual([], sot.tags)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


def resize_share(conn, share_id, share_size):
    # Be explicit about not wanting to use force if the share
    # will be extended.
    use_force = False
    print('Resize the share to the given size:')
    conn.share.resize_share(share_id, share_size, use_force)


def resize_shares_without_shrink(conn, min_size):
    # Sometimes, extending shares without shrinking
    # them (effectively setting a min size) is desirable.

    # Get list of shares from the connection.
    shares = conn.share.shares()

    # Loop over the shares:
    for share in shares:
        # Extend shares smaller than min_size to min_size,
        # but don't shrink shares larger than min_size.
        conn.share.resize_share(share.id, min_size, no_shrink=True)


def manage_share(conn, protocol, export_path, service_host, **params):
    # Manage a share with the given protocol, export path, service host, and
    # optional additional parameters
    managed_share = conn.share.manage_share(
        protocol, export_path, service_host, **params
    )

    # Can get the ID of the share, which is now being managed with Manila
    managed_share_id = managed_share.id
    print("The ID of the share which was managed: %s", managed_share_id)


def unmanage_share(conn, share_id):
    # Unmanage the share with the given share ID
    conn.share.unmanage_share(share_id)

    try:
        # Getting the share will raise an exception as it has been unmanaged
        conn.share.get_share(share_id)
    except Exception:
        pass



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from openstack.network.v2 import address_group as _address_group
from openstack.tests.functional import base


class TestAddressGroup(base.BaseFunctionalTest):
    ADDRESS_GROUP_ID = None
    ADDRESSES = ["10.0.0.1/32", "2001:db8::/32"]

    def setUp(self):
        super(TestAddressGroup, self).setUp()

        # Skip the tests if address group extension is not enabled.
        if not self.user_cloud.network.find_extension("address-group"):
            self.skipTest("Network Address Group extension disabled")

        self.ADDRESS_GROUP_NAME = self.getUniqueString()
        self.ADDRESS_GROUP_DESCRIPTION = self.getUniqueString()
        self.ADDRESS_GROUP_NAME_UPDATED = self.getUniqueString()
        self.ADDRESS_GROUP_DESCRIPTION_UPDATED = self.getUniqueString()
        address_group = self.user_cloud.network.create_address_group(
            name=self.ADDRESS_GROUP_NAME,
            description=self.ADDRESS_GROUP_DESCRIPTION,
            addresses=self.ADDRESSES,
        )
        assert isinstance(address_group, _address_group.AddressGroup)
        self.assertEqual(self.ADDRESS_GROUP_NAME, address_group.name)
        self.assertEqual(
            self.ADDRESS_GROUP_DESCRIPTION, address_group.description
        )
        self.assertCountEqual(self.ADDRESSES, address_group.addresses)
        self.ADDRESS_GROUP_ID = address_group.id

    def tearDown(self):
        sot = self.user_cloud.network.delete_address_group(
            self.ADDRESS_GROUP_ID
        )
        self.assertIsNone(sot)
        super(TestAddressGroup, self).tearDown()

    def test_find(self):
        sot = self.user_cloud.network.find_address_group(
            self.ADDRESS_GROUP_NAME
        )
        self.assertEqual(self.ADDRESS_GROUP_ID, sot.id)

    def test_get(self):
        sot = self.user_cloud.network.get_address_group(self.ADDRESS_GROUP_ID)
        self.assertEqual(self.ADDRESS_GROUP_NAME, sot.name)

    def test_list(self):
        names = [ag.name for ag in self.user_cloud.network.address_groups()]
        self.assertIn(self.ADDRESS_GROUP_NAME, names)

    def test_update(self):
        sot = self.user_cloud.network.update_address_group(
            self.ADDRESS_GROUP_ID,
            name=self.ADDRESS_GROUP_NAME_UPDATED,
            description=self.ADDRESS_GROUP_DESCRIPTION_UPDATED,
        )
        self.assertEqual(self.ADDRESS_GROUP_NAME_UPDATED, sot.name)
        self.assertEqual(
            self.ADDRESS_GROUP_DESCRIPTION_UPDATED, sot.description
        )

    def test_add_remove_addresses(self):
        addrs = ["127.0.0.1/32", "fe80::/10"]
        sot = self.user_cloud.network.add_addresses_to_address_group(
            self.ADDRESS_GROUP_ID, addrs
        )
        updated_addrs = self.ADDRESSES.copy()
        updated_addrs.extend(addrs)
        self.assertCountEqual(updated_addrs, sot.addresses)
        sot = self.user_cloud.network.remove_addresses_from_address_group(
            self.ADDRESS_GROUP_ID, addrs
        )
        self.assertCountEqual(self.ADDRESSES, sot.addresses)
